@@ -14,8 +14,6 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
-
-
 package org.apache.catalina.core;
 
 
@@ -24,6 +22,7 @@ import java.io.UnsupportedEncodingException;
 import java.nio.charset.Charset;
 import java.nio.charset.StandardCharsets;
 import java.util.ArrayList;
+import java.util.Arrays;
 import java.util.Collections;
 import java.util.Enumeration;
 import java.util.List;
@@ -278,8 +277,9 @@ class ApplicationHttpRequest extends HttpServletRequestWrapper {
     @Override
     public void removeAttribute(String name) {
 
-        if (!removeSpecial(name))
+        if (!removeSpecial(name)) {
             getRequest().removeAttribute(name);
+        }
 
     }
 
@@ -318,8 +318,9 @@ class ApplicationHttpRequest extends HttpServletRequestWrapper {
     @Override
     public RequestDispatcher getRequestDispatcher(String path) {
 
-        if (context == null)
+        if (context == null) {
             return null;
+        }
 
         if (path == null) {
             return null;
@@ -339,8 +340,9 @@ class ApplicationHttpRequest extends HttpServletRequestWrapper {
         // Convert a request-relative path to a context-relative one
         String servletPath =
             (String) getAttribute(RequestDispatcher.INCLUDE_SERVLET_PATH);
-        if (servletPath == null)
+        if (servletPath == null) {
             servletPath = getServletPath();
+        }
 
         // Add the path info, if there is any
         String pathInfo = getPathInfo();
@@ -540,8 +542,9 @@ class ApplicationHttpRequest extends HttpServletRequestWrapper {
         if (crossContext) {
 
             // There cannot be a session if no context has been assigned yet
-            if (context == null)
+            if (context == null) {
                 return null;
+            }
 
             // Return the current session if it exists and is valid
             if (session != null && session.isValid()) {
@@ -598,13 +601,16 @@ class ApplicationHttpRequest extends HttpServletRequestWrapper {
         if (crossContext) {
 
             String requestedSessionId = getRequestedSessionId();
-            if (requestedSessionId == null)
+            if (requestedSessionId == null) {
                 return false;
-            if (context == null)
+            }
+            if (context == null) {
                 return false;
+            }
             Manager manager = context.getManager();
-            if (manager == null)
+            if (manager == null) {
                 return false;
+            }
             Session session = null;
             try {
                 session = manager.findSession(requestedSessionId);
@@ -779,9 +785,10 @@ class ApplicationHttpRequest extends HttpServletRequestWrapper {
      */
     protected boolean isSpecial(String name) {
 
-        for (int i = 0; i < specials.length; i++) {
-            if (specials[i].equals(name))
+        for (String special : specials) {
+            if (special.equals(name)) {
                 return true;
+            }
         }
         return false;
 
@@ -849,17 +856,13 @@ class ApplicationHttpRequest extends HttpServletRequestWrapper {
         if (values1 == null) {
             // Skip - nothing to merge
         } else {
-            for (String value : values1) {
-                results.add(value);
-            }
+            results.addAll(Arrays.asList(values1));
         }
 
         if (values2 == null) {
             // Skip - nothing to merge
         } else {
-            for (String value : values2) {
-                results.add(value);
-            }
+            results.addAll(Arrays.asList(values2));
         }
 
         String values[] = new String[results.size()];
@@ -878,8 +881,9 @@ class ApplicationHttpRequest extends HttpServletRequestWrapper {
      */
     private void mergeParameters() {
 
-        if ((queryParamString == null) || (queryParamString.length() < 1))
+        if ((queryParamString == null) || (queryParamString.length() < 1)) {
             return;
+        }
 
         // Parse the query string from the dispatch target
         Parameters paramParser = new Parameters();
