@@ -182,7 +182,10 @@ public class StandardWrapper extends ContainerBase
 
     /**
      * Does this servlet implement the SingleThreadModel interface?
+     *
+     * @deprecated This will be removed in Tomcat 10.1 onwards.
      */
+    @Deprecated
     protected volatile boolean singleThreadModel = false;
 
 
@@ -194,19 +197,28 @@ public class StandardWrapper extends ContainerBase
 
     /**
      * Maximum number of STM instances.
+     *
+     * @deprecated This will be removed in Tomcat 10.1 onwards.
      */
+    @Deprecated
     protected int maxInstances = 20;
 
 
     /**
      * Number of instances currently loaded for a STM servlet.
+     *
+     * @deprecated This will be removed in Tomcat 10.1 onwards.
      */
+    @Deprecated
     protected int nInstances = 0;
 
 
     /**
      * Stack containing the STM instances.
+     *
+     * @deprecated This will be removed in Tomcat 10.1 onwards.
      */
+    @Deprecated
     protected Stack<Servlet> instancePool = null;
 
 
@@ -309,10 +321,11 @@ public class StandardWrapper extends ContainerBase
     @Override
     public void setAvailable(long available) {
         long oldAvailable = this.available;
-        if (available > System.currentTimeMillis())
+        if (available > System.currentTimeMillis()) {
             this.available = available;
-        else
+        } else {
             this.available = 0L;
+        }
         support.firePropertyChange("available", Long.valueOf(oldAvailable),
                                    Long.valueOf(this.available));
     }
@@ -335,7 +348,7 @@ public class StandardWrapper extends ContainerBase
     @Override
     public int getLoadOnStartup() {
 
-        if (isJspServlet && loadOnStartup < 0) {
+        if (isJspServlet && loadOnStartup == -1) {
             /*
              * JspServlet must always be preloaded, because its instance is
              * used during registerJMX (when registering the JSP
@@ -395,7 +408,10 @@ public class StandardWrapper extends ContainerBase
     /**
      * @return maximum number of instances that will be allocated when a single
      * thread model servlet is used.
+     *
+     * @deprecated This will be removed in Tomcat 10.1 onwards.
      */
+    @Deprecated
     public int getMaxInstances() {
         return this.maxInstances;
     }
@@ -406,7 +422,10 @@ public class StandardWrapper extends ContainerBase
      * thread model servlet is used.
      *
      * @param maxInstances New value of maxInstances
+     *
+     * @deprecated This will be removed in Tomcat 10.1 onwards.
      */
+    @Deprecated
     public void setMaxInstances(int maxInstances) {
 
         int oldMaxInstances = this.maxInstances;
@@ -426,9 +445,10 @@ public class StandardWrapper extends ContainerBase
     public void setParent(Container container) {
 
         if ((container != null) &&
-            !(container instanceof Context))
+            !(container instanceof Context)) {
             throw new IllegalArgumentException
                 (sm.getString("standardWrapper.notContext"));
+        }
         if (container instanceof StandardContext) {
             swallowOutput = ((StandardContext)container).getSwallowOutput();
             unloadDelay = ((StandardContext)container).getUnloadDelay();
@@ -514,7 +534,10 @@ public class StandardWrapper extends ContainerBase
      * @return {@code null} if the class has not been loaded, otherwise {@code
      *         true} if the servlet does implement {@code SingleThreadModel} and
      *         {@code false} if it does not.
+     *
+     * @deprecated This will be removed in Tomcat 10.1 onwards.
      */
+    @Deprecated
     public Boolean isSingleThreadModel() {
         // If the servlet has been loaded either singleThreadModel will be true
         // or instance will be non-null
@@ -531,15 +554,16 @@ public class StandardWrapper extends ContainerBase
     @Override
     public boolean isUnavailable() {
 
-        if (!isEnabled())
+        if (!isEnabled()) {
             return true;
-        else if (available == 0L)
+        } else if (available == 0L) {
             return false;
-        else if (available <= System.currentTimeMillis()) {
+        } else if (available <= System.currentTimeMillis()) {
             available = 0L;
             return false;
-        } else
+        } else {
             return true;
+        }
 
     }
 
@@ -616,8 +640,9 @@ public class StandardWrapper extends ContainerBase
     public void backgroundProcess() {
         super.backgroundProcess();
 
-        if (!getState().isAvailable())
+        if (!getState().isAvailable()) {
             return;
+        }
 
         if (getServlet() instanceof PeriodicEventListener) {
             ((PeriodicEventListener) getServlet()).periodicEvent();
@@ -639,8 +664,9 @@ public class StandardWrapper extends ContainerBase
         do {
             loops++;
             rootCauseCheck = rootCause.getCause();
-            if (rootCauseCheck != null)
+            if (rootCauseCheck != null) {
                 rootCause = rootCauseCheck;
+            }
         } while (rootCauseCheck != null && (loops < 20));
         return rootCause;
     }
@@ -695,8 +721,9 @@ public class StandardWrapper extends ContainerBase
         } finally {
             mappingsLock.writeLock().unlock();
         }
-        if(parent.getState().equals(LifecycleState.STARTED))
+        if(parent.getState().equals(LifecycleState.STARTED)) {
             fireContainerEvent(ADD_MAPPING_EVENT, mapping);
+        }
 
     }
 
@@ -1019,8 +1046,9 @@ public class StandardWrapper extends ContainerBase
     public synchronized Servlet loadServlet() throws ServletException {
 
         // Nothing to do if we already have an instance or an instance pool
-        if (!singleThreadModel && (instance != null))
+        if (!singleThreadModel && (instance != null)) {
             return instance;
+        }
 
         PrintStream out = System.out;
         if (swallowOutput) {
@@ -1111,7 +1139,9 @@ public class StandardWrapper extends ContainerBase
     private synchronized void initServlet(Servlet servlet)
             throws ServletException {
 
-        if (instanceInitialized && !singleThreadModel) return;
+        if (instanceInitialized && !singleThreadModel) {
+            return;
+        }
 
         // Call the initialization method of this servlet
         try {
@@ -1185,8 +1215,9 @@ public class StandardWrapper extends ContainerBase
         } finally {
             mappingsLock.writeLock().unlock();
         }
-        if(parent.getState().equals(LifecycleState.STARTED))
+        if(parent.getState().equals(LifecycleState.STARTED)) {
             fireContainerEvent(REMOVE_MAPPING_EVENT, mapping);
+        }
 
     }
 
@@ -1220,14 +1251,16 @@ public class StandardWrapper extends ContainerBase
     @Override
     public void unavailable(UnavailableException unavailable) {
         getServletContext().log(sm.getString("standardWrapper.unavailable", getName()));
-        if (unavailable == null)
+        if (unavailable == null) {
             setAvailable(Long.MAX_VALUE);
-        else if (unavailable.isPermanent())
+        } else if (unavailable.isPermanent()) {
             setAvailable(Long.MAX_VALUE);
-        else {
+        } else {
             int unavailableSeconds = unavailable.getUnavailableSeconds();
             if (unavailableSeconds <= 0)
+             {
                 unavailableSeconds = 60;        // Arbitrary default
+            }
             setAvailable(System.currentTimeMillis() +
                          (unavailableSeconds * 1000L));
         }
@@ -1249,8 +1282,9 @@ public class StandardWrapper extends ContainerBase
     public synchronized void unload() throws ServletException {
 
         // Nothing to do if we have never loaded the instance
-        if (!singleThreadModel && (instance == null))
+        if (!singleThreadModel && (instance == null)) {
             return;
+        }
         unloading = true;
 
         // Loaf a while if the current instance is allocated
@@ -1412,12 +1446,13 @@ public class StandardWrapper extends ContainerBase
      */
     @Override
     public ServletContext getServletContext() {
-        if (parent == null)
+        if (parent == null) {
             return null;
-        else if (!(parent instanceof Context))
+        } else if (!(parent instanceof Context)) {
             return null;
-        else
+        } else {
             return ((Context) parent).getServletContext();
+        }
     }
 
 
