@@ -14,8 +14,6 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
-
-
 package org.apache.catalina.core;
 
 
@@ -81,7 +79,7 @@ public final class StandardWrapperFacade
                 context = ((ApplicationContext) context).getFacade();
             }
         }
-        return (context);
+        return context;
     }
 
 
