@@ -75,8 +75,13 @@ import org.apache.tomcat.util.res.StringManager;
  * 'Access-Control-Request-Headers' header, for pre-flight request.</li>
  * </ul>
  *
- * @see <a href="http://www.w3.org/TR/cors/">CORS specification</a>
+ * If you extend this class and override one or more of the getXxx() methods,
+ * consider whether you also need to override
+ * {@link CorsFilter#doFilter(ServletRequest, ServletResponse, FilterChain)} and
+ * add appropriate locking so that the {@code doFilter()} method executes with a
+ * consistent configuration.
  *
+ * @see <a href="http://www.w3.org/TR/cors/">CORS specification</a>
  */
 public class CorsFilter extends GenericFilter {
 
@@ -152,7 +157,7 @@ public class CorsFilter extends GenericFilter {
         CorsFilter.CORSRequestType requestType = checkRequestType(request);
 
         // Adds CORS specific attributes to request.
-        if (decorateRequest) {
+        if (isDecorateRequest()) {
             CorsFilter.decorateCORSProperties(request, requestType);
         }
         switch (requestType) {
@@ -248,7 +253,7 @@ public class CorsFilter extends GenericFilter {
             return;
         }
 
-        if (!allowedHttpMethods.contains(method)) {
+        if (!getAllowedHttpMethods().contains(method)) {
             handleInvalidCORS(request, response, filterChain);
             return;
         }
@@ -310,7 +315,7 @@ public class CorsFilter extends GenericFilter {
         }
 
         // Section 6.2.5
-        if (!allowedHttpMethods.contains(accessControlRequestMethod)) {
+        if (!getAllowedHttpMethods().contains(accessControlRequestMethod)) {
             handleInvalidCORS(request, response, filterChain);
             return;
         }
@@ -318,7 +323,7 @@ public class CorsFilter extends GenericFilter {
         // Section 6.2.6
         if (!accessControlRequestHeaders.isEmpty()) {
             for (String header : accessControlRequestHeaders) {
-                if (!allowedHttpHeaders.contains(header)) {
+                if (!getAllowedHttpHeaders().contains(header)) {
                     handleInvalidCORS(request, response, filterChain);
                     return;
                 }
@@ -396,6 +401,9 @@ public class CorsFilter extends GenericFilter {
         final String method = request.getMethod();
         final String origin = request.getHeader(CorsFilter.REQUEST_HEADER_ORIGIN);
 
+        // Local copy to avoid concurrency issues if isAnyOriginAllowed()
+        // is overridden.
+        boolean anyOriginAllowed = isAnyOriginAllowed();
         if (!anyOriginAllowed) {
             // If only specific origins are allowed, the response will vary by
             // origin
@@ -420,13 +428,16 @@ public class CorsFilter extends GenericFilter {
         // If the resource supports credentials, add a single
         // Access-Control-Allow-Credentials header with the case-sensitive
         // string "true" as value.
-        if (supportsCredentials) {
+        if (isSupportsCredentials()) {
             response.addHeader(CorsFilter.RESPONSE_HEADER_ACCESS_CONTROL_ALLOW_CREDENTIALS, "true");
         }
 
         // If the list of exposed headers is not empty add one or more
         // Access-Control-Expose-Headers headers, with as values the header
         // field names given in the list of exposed headers.
+        // Local copy to avoid concurrency issues if getExposedHeaders()
+        // is overridden.
+        Collection<String> exposedHeaders = getExposedHeaders();
         if ((exposedHeaders != null) && (exposedHeaders.size() > 0)) {
             String exposedHeadersString = join(exposedHeaders, ",");
             response.addHeader(CorsFilter.RESPONSE_HEADER_ACCESS_CONTROL_EXPOSE_HEADERS,
@@ -446,19 +457,27 @@ public class CorsFilter extends GenericFilter {
             // non-CORS OPTIONS requests do not need to. The headers are always set
             // as a) they do no harm in the non-CORS case and b) it allows the same
             // response to be cached for CORS and non-CORS requests.
-
+            // Local copy to avoid concurrency issues if getPreflightMaxAge()
+            // is overridden.
+            long preflightMaxAge = getPreflightMaxAge();
             if (preflightMaxAge > 0) {
                 response.addHeader(
                         CorsFilter.RESPONSE_HEADER_ACCESS_CONTROL_MAX_AGE,
                         String.valueOf(preflightMaxAge));
             }
 
+            // Local copy to avoid concurrency issues if getAllowedHttpMethods()
+            // is overridden.
+            Collection<String> allowedHttpMethods = getAllowedHttpMethods();
             if  ((allowedHttpMethods != null) && (!allowedHttpMethods.isEmpty())) {
                 response.addHeader(
                         CorsFilter.RESPONSE_HEADER_ACCESS_CONTROL_ALLOW_METHODS,
                         join(allowedHttpMethods, ","));
             }
 
+            // Local copy to avoid concurrency issues if getAllowedHttpHeaders()
+            // is overridden.
+            Collection<String> allowedHttpHeaders = getAllowedHttpHeaders();
             if ((allowedHttpHeaders != null) && (!allowedHttpHeaders.isEmpty())) {
                 response.addHeader(
                         CorsFilter.RESPONSE_HEADER_ACCESS_CONTROL_ALLOW_HEADERS,
@@ -660,13 +679,13 @@ public class CorsFilter extends GenericFilter {
      *         otherwise.
      */
     private boolean isOriginAllowed(final String origin) {
-        if (anyOriginAllowed) {
+        if (isAnyOriginAllowed()) {
             return true;
         }
 
         // If 'Origin' header is a case-sensitive match of any of allowed
         // origins, then return true, else return false.
-        return allowedOrigins.contains(origin);
+        return getAllowedOrigins().contains(origin);
     }
 
 
@@ -688,7 +707,7 @@ public class CorsFilter extends GenericFilter {
      * @param preflightMaxAge
      *            The amount of seconds the user agent is allowed to cache the
      *            result of the pre-flight request.
-     * @throws ServletException
+     * @throws ServletException If the configuration is invalid
      */
     private void parseAndStore(final String allowedOrigins,
             final String allowedHttpMethods, final String allowedHttpHeaders,
@@ -864,6 +883,17 @@ public class CorsFilter extends GenericFilter {
     }
 
 
+    /**
+     * Should CORS specific attributes be added to the request.
+     *
+     * @return {@code true} if the request should be decorated, otherwise
+     *         {@code false}
+     */
+    public boolean isDecorateRequest() {
+        return decorateRequest;
+    }
+
+
     /*
      * Log objects are not Serializable but this Filter is because it extends
      * GenericFilter. Tomcat won't serialize a Filter but in case something else
