@@ -14,8 +14,6 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
-
-
 package org.apache.catalina.ha.backend;
 
 import org.apache.catalina.Lifecycle;
@@ -167,10 +165,11 @@ public class HeartbeatListener implements LifecycleListener {
 
         if (Lifecycle.PERIODIC_EVENT.equals(event.getType())) {
             if (sender == null) {
-                if (proxyList == null)
+                if (proxyList == null) {
                     sender = new MultiCastSender();
-                else
+                } else {
                     sender = new TcpSender();
+                }
             }
 
             /* Read busy and ready */
