@@ -91,7 +91,9 @@ public class BackupManager extends ClusterManagerBase
 
     @Override
     public ClusterMessage requestCompleted(String sessionId) {
-        if (!getState().isAvailable()) return null;
+        if (!getState().isAvailable()) {
+            return null;
+        }
         LazyReplicatedMap<String,Session> map =
                 (LazyReplicatedMap<String,Session>)sessions;
         map.replicate(sessionId,false);
@@ -143,7 +145,9 @@ public class BackupManager extends ClusterManagerBase
         super.startInternal();
 
         try {
-            if (cluster == null) throw new LifecycleException(sm.getString("backupManager.noCluster", getName()));
+            if (cluster == null) {
+                throw new LifecycleException(sm.getString("backupManager.noCluster", getName()));
+            }
             LazyReplicatedMap<String,Session> map = new LazyReplicatedMap<>(
                     this, cluster.getChannel(), rpcTimeout, getMapName(),
                     getClassLoaders(), terminateOnStartFailure);
@@ -159,7 +163,9 @@ public class BackupManager extends ClusterManagerBase
 
     public String getMapName() {
         String name = cluster.getManagerName(getName(),this)+"-"+"map";
-        if ( log.isDebugEnabled() ) log.debug("Backup manager, Setting map name to:"+name);
+        if ( log.isDebugEnabled() ) {
+            log.debug("Backup manager, Setting map name to:"+name);
+        }
         return name;
     }
 
@@ -177,8 +183,9 @@ public class BackupManager extends ClusterManagerBase
     @Override
     protected synchronized void stopInternal() throws LifecycleException {
 
-        if (log.isDebugEnabled())
+        if (log.isDebugEnabled()) {
             log.debug(sm.getString("backupManager.stopped", getName()));
+        }
 
         setState(LifecycleState.STOPPING);
 
