@@ -393,8 +393,9 @@ public class WebappLoader extends LifecycleMBeanBase
     @Override
     protected void startInternal() throws LifecycleException {
 
-        if (log.isDebugEnabled())
+        if (log.isDebugEnabled()) {
             log.debug(sm.getString("webappLoader.starting"));
+        }
 
         if (context.getResources() == null) {
             log.info(sm.getString("webappLoader.noResources", context));
@@ -446,8 +447,9 @@ public class WebappLoader extends LifecycleMBeanBase
     @Override
     protected void stopInternal() throws LifecycleException {
 
-        if (log.isDebugEnabled())
+        if (log.isDebugEnabled()) {
             log.debug(sm.getString("webappLoader.stopping"));
+        }
 
         setState(LifecycleState.STOPPING);
 
@@ -495,8 +497,9 @@ public class WebappLoader extends LifecycleMBeanBase
     public void propertyChange(PropertyChangeEvent event) {
 
         // Validate the source of this event
-        if (!(event.getSource() instanceof Context))
+        if (!(event.getSource() instanceof Context)) {
             return;
+        }
 
         // Process a relevant property change
         if (event.getPropertyName().equals("reloadable")) {
@@ -550,10 +553,12 @@ public class WebappLoader extends LifecycleMBeanBase
      */
     private void setPermissions() {
 
-        if (!Globals.IS_SECURITY_ENABLED)
+        if (!Globals.IS_SECURITY_ENABLED) {
             return;
-        if (context == null)
+        }
+        if (context == null) {
             return;
+        }
 
         // Tell the class loader the root of the context
         ServletContext servletContext = context.getServletContext();
@@ -587,11 +592,13 @@ public class WebappLoader extends LifecycleMBeanBase
     private void setClassPath() {
 
         // Validate our current state information
-        if (context == null)
+        if (context == null) {
             return;
+        }
         ServletContext servletContext = context.getServletContext();
-        if (servletContext == null)
+        if (servletContext == null) {
             return;
+        }
 
         StringBuilder classpath = new StringBuilder();
 
@@ -630,16 +637,19 @@ public class WebappLoader extends LifecycleMBeanBase
             URL repositories[] = ((URLClassLoader) loader).getURLs();
             for (URL url : repositories) {
                 String repository = url.toString();
-                if (repository.startsWith("file://"))
+                if (repository.startsWith("file://")) {
                     repository = UDecoder.URLDecode(repository.substring(7), StandardCharsets.UTF_8);
-                else if (repository.startsWith("file:"))
+                } else if (repository.startsWith("file:")) {
                     repository = UDecoder.URLDecode(repository.substring(5), StandardCharsets.UTF_8);
-                else
+                } else {
                     continue;
-                if (repository == null)
+                }
+                if (repository == null) {
                     continue;
-                if (classpath.length() > 0)
+                }
+                if (classpath.length() > 0) {
                     classpath.append(File.pathSeparator);
+                }
                 classpath.append(repository);
             }
         } else if (loader == ClassLoader.getSystemClassLoader()){
