@@ -14,8 +14,6 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
-
-
 package org.apache.catalina.manager;
 
 import java.io.File;
@@ -284,17 +282,16 @@ public final class HTMLManagerServlet extends ManagerServlet {
                     break;
                 }
 
-                if (isServiced(name)) {
-                    message = smClient.getString("managerServlet.inService", name);
-                } else {
-                    addServiced(name);
+                if (tryAddServiced(name)) {
                     try {
                         warPart.write(file.getAbsolutePath());
-                        // Perform new deployment
-                        check(name);
                     } finally {
                         removeServiced(name);
                     }
+                    // Perform new deployment
+                    check(name);
+                } else {
+                    message = smClient.getString("managerServlet.inService", name);
                 }
                 break;
             }
@@ -342,9 +339,10 @@ public final class HTMLManagerServlet extends ManagerServlet {
                      String message,
                      StringManager smClient) throws IOException {
 
-        if (debug >= 1)
+        if (debug >= 1) {
             log("list: Listing contexts for virtual host '" +
                 host.getName() + "'");
+        }
 
         PrintWriter writer = response.getWriter();
 
@@ -404,8 +402,9 @@ public final class HTMLManagerServlet extends ManagerServlet {
         // Create sorted map of deployed applications by context name.
         Container children[] = host.findChildren();
         String contextNames[] = new String[children.length];
-        for (int i = 0; i < children.length; i++)
+        for (int i = 0; i < children.length; i++) {
             contextNames[i] = children[i].getName();
+        }
 
         Arrays.sort(contextNames);
 
@@ -798,7 +797,7 @@ public final class HTMLManagerServlet extends ManagerServlet {
      */
     @Override
     public String getServletInfo() {
-        return "HTMLManagerServlet, Copyright (c) 1999-2021, The Apache Software Foundation";
+        return "HTMLManagerServlet, Copyright (c) 1999-2022, The Apache Software Foundation";
     }
 
     /**
@@ -921,7 +920,9 @@ public final class HTMLManagerServlet extends ManagerServlet {
             StringManager smClient) {
 
         List<Session> sessions = getSessionsForName(cn, smClient);
-        if (sessions.isEmpty()) return null;
+        if (sessions.isEmpty()) {
+            return null;
+        }
         for(Session session : sessions) {
             if (session.getId().equals(id)) {
                 return session;
