@@ -131,7 +131,7 @@ public class MapperListener extends LifecycleMBeanBase
     @Override
     protected String getDomainInternal() {
         if (service instanceof LifecycleMBeanBase) {
-            return ((LifecycleMBeanBase) service).getDomain();
+            return service.getDomain();
         } else {
             return null;
         }
@@ -141,7 +141,7 @@ public class MapperListener extends LifecycleMBeanBase
     @Override
     protected String getObjectNameKeyProperties() {
         // Same as connector but Mapper rather than Connector
-        return ("type=Mapper");
+        return "type=Mapper";
     }
 
     // --------------------------------------------- Container Listener methods
@@ -266,7 +266,7 @@ public class MapperListener extends LifecycleMBeanBase
 
         boolean found = false;
 
-        if (defaultHost != null && defaultHost.length() >0) {
+        if (defaultHost != null && defaultHost.length() > 0) {
             Container[] containers = engine.findChildren();
 
             for (Container container : containers) {
@@ -286,11 +286,10 @@ public class MapperListener extends LifecycleMBeanBase
             }
         }
 
-        if(found) {
+        if (found) {
             mapper.setDefaultHostName(defaultHost);
         } else {
-            log.warn(sm.getString("mapperListener.unknownDefaultHost",
-                    defaultHost, service));
+            log.error(sm.getString("mapperListener.unknownDefaultHost", defaultHost, service));
         }
     }
 
@@ -308,6 +307,10 @@ public class MapperListener extends LifecycleMBeanBase
                 registerContext((Context) container);
             }
         }
+
+        // Default host may have changed
+        findDefaultHost();
+
         if(log.isDebugEnabled()) {
             log.debug(sm.getString("mapperListener.registerHost",
                     host.getName(), domain, service));
@@ -324,6 +327,9 @@ public class MapperListener extends LifecycleMBeanBase
 
         mapper.removeHost(hostname);
 
+        // Default host may have changed
+        findDefaultHost();
+
         if(log.isDebugEnabled()) {
             log.debug(sm.getString("mapperListener.unregisterHost", hostname,
                     domain, service));
@@ -448,13 +454,9 @@ public class MapperListener extends LifecycleMBeanBase
         }
     }
 
-    /**
+    /*
      * Populate <code>wrappers</code> list with information for registration of
      * mappings for this wrapper in this context.
-     *
-     * @param context
-     * @param wrapper
-     * @param wrappers
      */
     private void prepareWrapperMappingInfo(Context context, Wrapper wrapper,
             List<WrapperMappingInfo> wrappers) {
@@ -506,7 +508,8 @@ public class MapperListener extends LifecycleMBeanBase
     /**
      * Add this mapper to the container and all child containers
      *
-     * @param container
+     * @param container the container (and any associated childern) to which
+     *        the mapper is to be added
      */
     private void addListeners(Container container) {
         container.addContainerListener(this);
@@ -520,7 +523,8 @@ public class MapperListener extends LifecycleMBeanBase
     /**
      * Remove this mapper from the container and all child containers
      *
-     * @param container
+     * @param container the container (and any associated childern) from which
+     *        the mapper is to be removed
      */
     private void removeListeners(Container container) {
         container.removeContainerListener(this);
