@@ -14,13 +14,9 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
-
 package org.apache.catalina.mbeans;
 
-import javax.management.InstanceNotFoundException;
 import javax.management.MBeanException;
-import javax.management.RuntimeOperationsException;
-import javax.management.modelmbean.InvalidTargetObjectTypeException;
 
 import org.apache.catalina.Context;
 import org.apache.tomcat.util.descriptor.web.ApplicationParameter;
@@ -29,12 +25,7 @@ import org.apache.tomcat.util.descriptor.web.FilterDef;
 import org.apache.tomcat.util.descriptor.web.FilterMap;
 import org.apache.tomcat.util.descriptor.web.SecurityConstraint;
 
-public class ContextMBean extends ContainerMBean {
-
-    public ContextMBean() throws MBeanException, RuntimeOperationsException {
-
-        super();
-    }
+public class ContextMBean extends BaseCatalinaMBean<Context> {
 
      /**
      * Return the set of application parameters for this application.
@@ -43,27 +34,18 @@ public class ContextMBean extends ContainerMBean {
      */
     public String[] findApplicationParameters() throws MBeanException {
 
-        Context context;
-        try {
-            context = (Context)getManagedResource();
-        } catch (InstanceNotFoundException e) {
-            throw new MBeanException(e);
-        } catch (RuntimeOperationsException e) {
-            throw new MBeanException(e);
-        } catch (InvalidTargetObjectTypeException e) {
-            throw new MBeanException(e);
-        }
+        Context context = doGetManagedResource();
 
         ApplicationParameter[] params = context.findApplicationParameters();
         String[] stringParams = new String[params.length];
-        for(int counter=0; counter < params.length; counter++){
-           stringParams[counter]=params[counter].toString();
+        for (int counter = 0; counter < params.length; counter++) {
+           stringParams[counter] = params[counter].toString();
         }
 
         return stringParams;
-
     }
 
+
     /**
      * Return the security constraints for this web application.
      * If there are none, a zero-length array is returned.
@@ -73,27 +55,18 @@ public class ContextMBean extends ContainerMBean {
      */
     public String[] findConstraints() throws MBeanException {
 
-        Context context;
-        try {
-            context = (Context)getManagedResource();
-        } catch (InstanceNotFoundException e) {
-            throw new MBeanException(e);
-        } catch (RuntimeOperationsException e) {
-            throw new MBeanException(e);
-        } catch (InvalidTargetObjectTypeException e) {
-            throw new MBeanException(e);
-        }
+        Context context = doGetManagedResource();
 
         SecurityConstraint[] constraints = context.findConstraints();
         String[] stringConstraints = new String[constraints.length];
-        for(int counter=0; counter < constraints.length; counter++){
-            stringConstraints[counter]=constraints[counter].toString();
+        for (int counter = 0; counter < constraints.length; counter++) {
+            stringConstraints[counter] = constraints[counter].toString();
         }
 
         return stringConstraints;
-
     }
 
+
     /**
      * Return the error page entry for the specified HTTP error code,
      * if any; otherwise return <code>null</code>.
@@ -103,22 +76,28 @@ public class ContextMBean extends ContainerMBean {
      * @throws MBeanException propagated from the managed resource access
      */
     public String findErrorPage(int errorCode) throws MBeanException {
-
-        Context context;
-        try {
-            context = (Context)getManagedResource();
-        } catch (InstanceNotFoundException e) {
-            throw new MBeanException(e);
-        } catch (RuntimeOperationsException e) {
-            throw new MBeanException(e);
-        } catch (InvalidTargetObjectTypeException e) {
-            throw new MBeanException(e);
-        }
-
+        Context context = doGetManagedResource();
         return context.findErrorPage(errorCode).toString();
-
     }
 
+
+    /**
+     * Return the error page entry for the specified Java exception type,
+     * if any; otherwise return <code>null</code>.
+     *
+     * @param exceptionType Exception type to look up
+     * @return a string representation of the error page
+     * @throws MBeanException propagated from the managed resource access
+     * @deprecated Unused. Will be removed in Tomcat 10.
+     *             Use {@link #findErrorPage(Throwable)} instead.
+     */
+    @Deprecated
+    public String findErrorPage(String exceptionType) throws MBeanException {
+        Context context = doGetManagedResource();
+        return context.findErrorPage(exceptionType).toString();
+    }
+
+
     /**
      * Return the error page entry for the specified Java exception type,
      * if any; otherwise return <code>null</code>.
@@ -127,23 +106,12 @@ public class ContextMBean extends ContainerMBean {
      * @return a string representation of the error page
      * @throws MBeanException propagated from the managed resource access
      */
-    public String findErrorPage(String exceptionType) throws MBeanException {
-
-        Context context;
-        try {
-            context = (Context)getManagedResource();
-        } catch (InstanceNotFoundException e) {
-            throw new MBeanException(e);
-        } catch (RuntimeOperationsException e) {
-            throw new MBeanException(e);
-        } catch (InvalidTargetObjectTypeException e) {
-            throw new MBeanException(e);
-        }
-
+    public String findErrorPage(Throwable exceptionType) throws MBeanException {
+        Context context = doGetManagedResource();
         return context.findErrorPage(exceptionType).toString();
-
     }
 
+
     /**
      * Return the set of defined error pages for all specified error codes
      * and exception types.
@@ -152,27 +120,18 @@ public class ContextMBean extends ContainerMBean {
      */
     public String[] findErrorPages() throws MBeanException {
 
-        Context context;
-        try {
-            context = (Context)getManagedResource();
-        } catch (InstanceNotFoundException e) {
-            throw new MBeanException(e);
-        } catch (RuntimeOperationsException e) {
-            throw new MBeanException(e);
-        } catch (InvalidTargetObjectTypeException e) {
-            throw new MBeanException(e);
-        }
+        Context context = doGetManagedResource();
 
         ErrorPage[] pages = context.findErrorPages();
         String[] stringPages = new String[pages.length];
-        for(int counter=0; counter < pages.length; counter++){
-            stringPages[counter]=pages[counter].toString();
+        for (int counter = 0; counter < pages.length; counter++) {
+            stringPages[counter] = pages[counter].toString();
         }
 
         return stringPages;
-
     }
 
+
     /**
      * Return the filter definition for the specified filter name, if any;
      * otherwise return <code>null</code>.
@@ -183,22 +142,13 @@ public class ContextMBean extends ContainerMBean {
      */
     public String findFilterDef(String name) throws MBeanException {
 
-        Context context;
-        try {
-            context = (Context)getManagedResource();
-        } catch (InstanceNotFoundException e) {
-            throw new MBeanException(e);
-        } catch (RuntimeOperationsException e) {
-            throw new MBeanException(e);
-        } catch (InvalidTargetObjectTypeException e) {
-            throw new MBeanException(e);
-        }
+        Context context = doGetManagedResource();
 
         FilterDef filterDef = context.findFilterDef(name);
         return filterDef.toString();
-
     }
 
+
     /**
      * Return the set of defined filters for this Context.
      * @return a string array with a representation of all
@@ -207,27 +157,18 @@ public class ContextMBean extends ContainerMBean {
      */
     public String[] findFilterDefs() throws MBeanException {
 
-        Context context;
-        try {
-            context = (Context)getManagedResource();
-        } catch (InstanceNotFoundException e) {
-            throw new MBeanException(e);
-        } catch (RuntimeOperationsException e) {
-            throw new MBeanException(e);
-        } catch (InvalidTargetObjectTypeException e) {
-            throw new MBeanException(e);
-        }
+        Context context = doGetManagedResource();
 
         FilterDef[] filterDefs = context.findFilterDefs();
         String[] stringFilters = new String[filterDefs.length];
-        for(int counter=0; counter < filterDefs.length; counter++){
-            stringFilters[counter]=filterDefs[counter].toString();
+        for (int counter = 0; counter < filterDefs.length; counter++) {
+            stringFilters[counter] = filterDefs[counter].toString();
         }
 
         return stringFilters;
-
     }
 
+
     /**
      * Return the set of filter mappings for this Context.
      * @return a string array with a representation of all the filter mappings
@@ -235,25 +176,14 @@ public class ContextMBean extends ContainerMBean {
      */
     public String[] findFilterMaps() throws MBeanException {
 
-        Context context;
-        try {
-            context = (Context)getManagedResource();
-        } catch (InstanceNotFoundException e) {
-            throw new MBeanException(e);
-        } catch (RuntimeOperationsException e) {
-            throw new MBeanException(e);
-        } catch (InvalidTargetObjectTypeException e) {
-            throw new MBeanException(e);
-        }
+        Context context = doGetManagedResource();
 
         FilterMap[] maps = context.findFilterMaps();
         String[] stringMaps = new String[maps.length];
-        for(int counter=0; counter < maps.length; counter++){
-            stringMaps[counter]=maps[counter].toString();
+        for (int counter = 0; counter < maps.length; counter++) {
+            stringMaps[counter] = maps[counter].toString();
         }
 
         return stringMaps;
-
     }
-
 }
