@@ -446,22 +446,28 @@ public class MBeanFactory {
                                              pname.getKeyProperty("host"));
         if(mserver.isRegistered(deployer)) {
             String contextName = context.getName();
-            mserver.invoke(deployer, "addServiced",
-                           new Object [] {contextName},
-                           new String [] {"java.lang.String"});
-            String configPath = (String)mserver.getAttribute(deployer,
-                                                             "configBaseName");
-            String baseName = context.getBaseName();
-            File configFile = new File(new File(configPath), baseName+".xml");
-            if (configFile.isFile()) {
-                context.setConfigFile(configFile.toURI().toURL());
+            Boolean result = (Boolean) mserver.invoke(deployer, "tryAddServiced",
+                    new Object [] {contextName},
+                    new String [] {"java.lang.String"});
+            if (result.booleanValue()) {
+                try {
+                    String configPath = (String)mserver.getAttribute(deployer, "configBaseName");
+                    String baseName = context.getBaseName();
+                    File configFile = new File(new File(configPath), baseName+".xml");
+                    if (configFile.isFile()) {
+                        context.setConfigFile(configFile.toURI().toURL());
+                    }
+                    mserver.invoke(deployer, "manageApp",
+                            new Object[] {context},
+                            new String[] {"org.apache.catalina.Context"});
+                } finally {
+                    mserver.invoke(deployer, "removeServiced",
+                            new Object [] {contextName},
+                            new String [] {"java.lang.String"});
+                }
+            } else {
+                throw new IllegalStateException(sm.getString("mBeanFactory.contextCreate.addServicedFail", contextName));
             }
-            mserver.invoke(deployer, "manageApp",
-                           new Object[] {context},
-                           new String[] {"org.apache.catalina.Context"});
-            mserver.invoke(deployer, "removeServiced",
-                           new Object [] {contextName},
-                           new String [] {"java.lang.String"});
         } else {
             log.warn(sm.getString("mBeanFactory.noDeployer", pname.getKeyProperty("host")));
             Service service = getService(pname);
@@ -744,27 +750,35 @@ public class MBeanFactory {
                                              hostName);
         String pathStr = getPathStr(path);
         if(mserver.isRegistered(deployer)) {
-            mserver.invoke(deployer,"addServiced",
-                           new Object[]{pathStr},
-                           new String[] {"java.lang.String"});
-            mserver.invoke(deployer,"unmanageApp",
-                           new Object[] {pathStr},
-                           new String[] {"java.lang.String"});
-            mserver.invoke(deployer,"removeServiced",
-                           new Object[] {pathStr},
-                           new String[] {"java.lang.String"});
+            Boolean result = (Boolean) mserver.invoke(deployer,"tryAddServiced",
+                       new Object[]{pathStr},
+                       new String[] {"java.lang.String"});
+            if (result.booleanValue()) {
+                try {
+                    mserver.invoke(deployer,"unmanageApp",
+                            new Object[] {pathStr},
+                            new String[] {"java.lang.String"});
+                } finally {
+                    mserver.invoke(deployer,"removeServiced",
+                            new Object[] {pathStr},
+                            new String[] {"java.lang.String"});
+                }
+            } else {
+                throw new IllegalStateException(sm.getString("mBeanFactory.removeContext.addServicedFail", pathStr));
+            }
         } else {
             log.warn(sm.getString("mBeanFactory.noDeployer", hostName));
             Host host = (Host) engine.findChild(hostName);
             Context context = (Context) host.findChild(pathStr);
             // Remove this component from its parent component
             host.removeChild(context);
-            if(context instanceof StandardContext)
-            try {
-                context.destroy();
-            } catch (Exception e) {
-                log.warn(sm.getString("mBeanFactory.contextDestroyError"), e);
-           }
+            if(context instanceof StandardContext) {
+                try {
+                    context.destroy();
+                } catch (Exception e) {
+                    log.warn(sm.getString("mBeanFactory.contextDestroyError"), e);
+         }
+            }
 
         }
     }
