@@ -297,9 +297,10 @@ public class DataSourceRealm extends RealmBase {
         // No user or no credentials
         // Can't possibly authenticate, don't bother the database then
         if (username == null || credentials == null) {
-            if (containerLog.isTraceEnabled())
+            if (containerLog.isTraceEnabled()) {
                 containerLog.trace(sm.getString("dataSourceRealm.authenticateFailure",
                                                 username));
+            }
             return null;
         }
 
@@ -311,9 +312,10 @@ public class DataSourceRealm extends RealmBase {
             // Waste a bit of time as not to reveal that the user does not exist.
             getCredentialHandler().mutate(credentials);
 
-            if (containerLog.isTraceEnabled())
+            if (containerLog.isTraceEnabled()) {
                 containerLog.trace(sm.getString("dataSourceRealm.authenticateFailure",
                                                 username));
+            }
             return null;
         }
 
@@ -321,13 +323,15 @@ public class DataSourceRealm extends RealmBase {
         boolean validated = getCredentialHandler().matches(credentials, dbCredentials);
 
         if (validated) {
-            if (containerLog.isTraceEnabled())
+            if (containerLog.isTraceEnabled()) {
                 containerLog.trace(sm.getString("dataSourceRealm.authenticateSuccess",
                                                 username));
+            }
         } else {
-            if (containerLog.isTraceEnabled())
+            if (containerLog.isTraceEnabled()) {
                 containerLog.trace(sm.getString("dataSourceRealm.authenticateFailure",
                                                 username));
+            }
             return null;
         }
 
@@ -346,8 +350,9 @@ public class DataSourceRealm extends RealmBase {
     protected void close(Connection dbConnection) {
 
         // Do nothing if the database connection is already closed
-        if (dbConnection == null)
+        if (dbConnection == null) {
             return;
+        }
 
         // Commit if not auto committed
         try {
@@ -355,7 +360,7 @@ public class DataSourceRealm extends RealmBase {
                 dbConnection.commit();
             }
         } catch (SQLException e) {
-            containerLog.error("Exception committing connection before closing:", e);
+            containerLog.error(sm.getString("dataSourceRealm.commit"), e);
         }
 
         // Close this database connection, and log any errors
