@@ -87,14 +87,20 @@ public class ExpressionParseTree {
             return;
         }
         while (true) {
-            if (oppStack.size() == 0) break;
+            if (oppStack.size() == 0) {
+                break;
+            }
             OppNode top = oppStack.get(0);
             // If the top is a spacer then don't pop
             // anything
-            if (top == null) break;
+            if (top == null) {
+                break;
+            }
             // If the top node has a lower precedence then
             // let it stay
-            if (top.getPrecedence() < node.getPrecedence()) break;
+            if (top.getPrecedence() < node.getPrecedence()) {
+                break;
+            }
             // Remove the top node
             oppStack.remove(0);
             // Let it fill its branches
@@ -135,8 +141,9 @@ public class ExpressionParseTree {
         ExpressionTokenizer et = new ExpressionTokenizer(expr);
         while (et.hasMoreTokens()) {
             int token = et.nextToken();
-            if (token != ExpressionTokenizer.TOKEN_STRING)
+            if (token != ExpressionTokenizer.TOKEN_STRING) {
                 currStringNode = null;
+            }
             switch (token) {
                 case ExpressionTokenizer.TOKEN_STRING :
                     if (currStringNode == null) {
@@ -238,8 +245,9 @@ public class ExpressionParseTree {
          * @return the value string
          */
         public String getValue() {
-            if (resolved == null)
+            if (resolved == null) {
                 resolved = ssiMediator.substituteVariables(value.toString());
+            }
             return resolved;
         }
 
@@ -325,8 +333,9 @@ public class ExpressionParseTree {
     private static final class AndNode extends OppNode {
         @Override
         public boolean evaluate() {
-            if (!left.evaluate()) // Short circuit
+            if (!left.evaluate()) {
                 return false;
+            }
             return right.evaluate();
         }
 
@@ -345,8 +354,9 @@ public class ExpressionParseTree {
     private static final class OrNode extends OppNode {
         @Override
         public boolean evaluate() {
-            if (left.evaluate()) // Short circuit
+            if (left.evaluate()) {
                 return true;
+            }
             return right.evaluate();
         }
 
