@@ -203,7 +203,7 @@ public class SSIServletExternalResolver implements SSIExternalResolver {
                 }
                 if (accept != null) {
                     Enumeration<String> acceptHeaders = req.getHeaders(accept);
-                    if (acceptHeaders != null)
+                    if (acceptHeaders != null) {
                         if (acceptHeaders.hasMoreElements()) {
                             StringBuilder rv = new StringBuilder(
                                     acceptHeaders.nextElement());
@@ -213,6 +213,7 @@ public class SSIServletExternalResolver implements SSIExternalResolver {
                             }
                         retVal = rv.toString();
                     }
+                    }
                 }
             }
             else if (nameParts[1].equals("CONNECTION")) {
@@ -224,12 +225,14 @@ public class SSIServletExternalResolver implements SSIExternalResolver {
             else if (nameParts[1].equals("REFERER")) {
                 retVal = req.getHeader("Referer");
             }
-            else if (nameParts[1].equals("USER"))
-                if (nameParts.length == 3)
+            else if (nameParts[1].equals("USER")) {
+                if (nameParts.length == 3) {
                     if (nameParts[2].equals("AGENT")) {
                         requiredParts = 3;
                         retVal = req.getHeader("User-Agent");
                     }
+                }
+            }
 
         } else if (nameParts[0].equals("PATH")) {
             if (nameParts[1].equals("INFO")) {
@@ -299,7 +302,9 @@ public class SSIServletExternalResolver implements SSIExternalResolver {
                 // If this is an error page, get the original URI
                 retVal = (String) req.getAttribute(
                         RequestDispatcher.FORWARD_REQUEST_URI);
-                if (retVal == null) retVal=req.getRequestURI();
+                if (retVal == null) {
+                    retVal=req.getRequestURI();
+                }
             }
         } else if (nameParts[0].equals("SCRIPT")) {
             String scriptName = req.getServletPath();
@@ -332,7 +337,9 @@ public class SSIServletExternalResolver implements SSIExternalResolver {
         } else if (name.equalsIgnoreCase("UNIQUE_ID")) {
             retVal = req.getRequestedSessionId();
         }
-        if (requiredParts != nameParts.length) return null;
+        if (requiredParts != nameParts.length) {
+            return null;
+        }
             return retVal;
     }
 
