@@ -14,14 +14,10 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
-
-
 package org.apache.catalina.startup;
 
-
 import org.apache.tomcat.util.digester.Digester;
-import org.apache.tomcat.util.digester.RuleSetBase;
-
+import org.apache.tomcat.util.digester.RuleSet;
 
 /**
  * <p><strong>RuleSet</strong> for processing the contents of a
@@ -31,13 +27,10 @@ import org.apache.tomcat.util.digester.RuleSetBase;
  *
  * @author Craig R. McClanahan
  */
-@SuppressWarnings("deprecation")
-public class EngineRuleSet extends RuleSetBase {
-
+public class EngineRuleSet implements RuleSet {
 
     // ----------------------------------------------------- Instance Variables
 
-
     /**
      * The matching pattern prefix to use for recognizing our elements.
      */
@@ -46,7 +39,6 @@ public class EngineRuleSet extends RuleSetBase {
 
     // ------------------------------------------------------------ Constructor
 
-
     /**
      * Construct an instance of this <code>RuleSet</code> with the default
      * matching pattern prefix.
@@ -70,7 +62,6 @@ public class EngineRuleSet extends RuleSetBase {
 
     // --------------------------------------------------------- Public Methods
 
-
     /**
      * <p>Add the set of Rule instances defined in this RuleSet to the
      * specified <code>Digester</code> instance, associating them with
@@ -123,8 +114,5 @@ public class EngineRuleSet extends RuleSetBase {
         digester.addSetNext(prefix + "Engine/Valve",
                             "addValve",
                             "org.apache.catalina.Valve");
-
     }
-
-
 }
