@@ -14,8 +14,6 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
-
-
 package org.apache.catalina.startup;
 
 
@@ -204,7 +202,9 @@ public final class UserConfig
      * @return the regular expression used to test for user who deployment is allowed.
      */
     public String getAllow() {
-        if (allow == null) return null;
+        if (allow == null) {
+            return null;
+        }
         return allow.toString();
     }
 
@@ -227,7 +227,9 @@ public final class UserConfig
      * @return the regular expression used to test for user who deployment is denied.
      */
     public String getDeny() {
-        if (deny == null) return null;
+        if (deny == null) {
+            return null;
+        }
         return deny.toString();
     }
 
@@ -265,10 +267,11 @@ public final class UserConfig
         }
 
         // Process the event that has occurred
-        if (event.getType().equals(Lifecycle.START_EVENT))
+        if (event.getType().equals(Lifecycle.START_EVENT)) {
             start();
-        else if (event.getType().equals(Lifecycle.STOP_EVENT))
+        } else if (event.getType().equals(Lifecycle.STOP_EVENT)) {
             stop();
+        }
 
     }
 
@@ -282,8 +285,9 @@ public final class UserConfig
      */
     private void deploy() {
 
-        if (host.getLogger().isDebugEnabled())
+        if (host.getLogger().isDebugEnabled()) {
             host.getLogger().debug(sm.getString("userConfig.deploying"));
+        }
 
         // Load the user database object for this host
         UserDatabase database = null;
@@ -303,7 +307,9 @@ public final class UserConfig
         Enumeration<String> users = database.getUsers();
         while (users.hasMoreElements()) {
             String user = users.nextElement();
-            if (!isDeployAllowed(user)) continue;
+            if (!isDeployAllowed(user)) {
+                continue;
+            }
             String home = database.getHome(user);
             results.add(executor.submit(new DeployUserDirectory(this, user, home)));
         }
@@ -329,11 +335,13 @@ public final class UserConfig
 
         // Does this user have a web application to be deployed?
         String contextPath = "/~" + user;
-        if (host.findChild(contextPath) != null)
+        if (host.findChild(contextPath) != null) {
             return;
+        }
         File app = new File(home, directoryName);
-        if (!app.exists() || !app.isDirectory())
+        if (!app.exists() || !app.isDirectory()) {
             return;
+        }
 
         host.getLogger().info(sm.getString("userConfig.deploy", user));
 
@@ -359,8 +367,9 @@ public final class UserConfig
      */
     private void start() {
 
-        if (host.getLogger().isDebugEnabled())
+        if (host.getLogger().isDebugEnabled()) {
             host.getLogger().debug(sm.getString("userConfig.start"));
+        }
 
         deploy();
 
@@ -372,8 +381,9 @@ public final class UserConfig
      */
     private void stop() {
 
-        if (host.getLogger().isDebugEnabled())
+        if (host.getLogger().isDebugEnabled()) {
             host.getLogger().debug(sm.getString("userConfig.stop"));
+        }
 
     }
 
