@@ -201,11 +201,15 @@ public class GroupChannel extends ChannelInterceptorBase
         super.heartbeat();
 
         for (MembershipListener listener : membershipListeners) {
-            if ( listener instanceof Heartbeat ) ((Heartbeat)listener).heartbeat();
+            if ( listener instanceof Heartbeat ) {
+                ((Heartbeat)listener).heartbeat();
+            }
         }
 
         for (ChannelListener listener : channelListeners) {
-            if ( listener instanceof Heartbeat ) ((Heartbeat)listener).heartbeat();
+            if ( listener instanceof Heartbeat ) {
+                ((Heartbeat)listener).heartbeat();
+            }
         }
     }
 
@@ -244,7 +248,9 @@ public class GroupChannel extends ChannelInterceptorBase
     @Override
     public UniqueId send(Member[] destination, Serializable msg, int options, ErrorHandler handler)
             throws ChannelException {
-        if ( msg == null ) throw new ChannelException(sm.getString("groupChannel.nullMessage"));
+        if ( msg == null ) {
+            throw new ChannelException(sm.getString("groupChannel.nullMessage"));
+        }
         XByteBuffer buffer = null;
         try {
             if (destination == null || destination.length == 0) {
@@ -284,7 +290,9 @@ public class GroupChannel extends ChannelInterceptorBase
         } catch (RuntimeException | IOException e) {
             throw new ChannelException(e);
         } finally {
-            if ( buffer != null ) BufferPool.getBufferPool().returnBuffer(buffer);
+            if ( buffer != null ) {
+                BufferPool.getBufferPool().returnBuffer(buffer);
+            }
         }
     }
 
@@ -300,7 +308,9 @@ public class GroupChannel extends ChannelInterceptorBase
      */
     @Override
     public void messageReceived(ChannelMessage msg) {
-        if ( msg == null ) return;
+        if ( msg == null ) {
+            return;
+        }
         try {
             if ( Logs.MESSAGES.isTraceEnabled() ) {
                 Logs.MESSAGES.trace("GroupChannel - Received msg:" +
@@ -336,7 +346,9 @@ public class GroupChannel extends ChannelInterceptorBase
                     delivered = true;
                     //if the message was accepted by an RPC channel, that channel
                     //is responsible for returning the reply, otherwise we send an absence reply
-                    if (channelListener instanceof RpcChannel) rx = true;
+                    if (channelListener instanceof RpcChannel) {
+                        rx = true;
+                    }
                 }
             }//for
             if ((!rx) && (fwd instanceof RpcMessage)) {
@@ -352,7 +364,9 @@ public class GroupChannel extends ChannelInterceptorBase
         } catch ( Exception x ) {
             //this could be the channel listener throwing an exception, we should log it
             //as a warning.
-            if ( log.isWarnEnabled() ) log.warn(sm.getString("groupChannel.receiving.error"),x);
+            if ( log.isWarnEnabled() ) {
+                log.warn(sm.getString("groupChannel.receiving.error"),x);
+            }
             throw new RemoteProcessException(sm.getString("groupChannel.receiving.error"),x);
         }
     }
@@ -367,7 +381,9 @@ public class GroupChannel extends ChannelInterceptorBase
     protected void sendNoRpcChannelReply(RpcMessage msg, Member destination) {
         try {
             //avoid circular loop
-            if ( msg instanceof RpcMessage.NoRpcChannelReply) return;
+            if ( msg instanceof RpcMessage.NoRpcChannelReply) {
+                return;
+            }
             RpcMessage.NoRpcChannelReply reply =
                     new RpcMessage.NoRpcChannelReply(msg.rpcId, msg.uuid);
             send(new Member[]{destination},reply,Channel.SEND_OPTIONS_ASYNCHRONOUS);
@@ -385,7 +401,9 @@ public class GroupChannel extends ChannelInterceptorBase
     public void memberAdded(Member member) {
         //notify upwards
         for (MembershipListener membershipListener : membershipListeners) {
-            if (membershipListener != null) membershipListener.memberAdded(member);
+            if (membershipListener != null) {
+                membershipListener.memberAdded(member);
+            }
         }
     }
 
@@ -398,7 +416,9 @@ public class GroupChannel extends ChannelInterceptorBase
     public void memberDisappeared(Member member) {
         //notify upwards
         for (MembershipListener membershipListener : membershipListeners) {
-            if (membershipListener != null) membershipListener.memberDisappeared(member);
+            if (membershipListener != null) {
+                membershipListener.memberDisappeared(member);
+            }
         }
     }
 
@@ -450,8 +470,10 @@ public class GroupChannel extends ChannelInterceptorBase
             }//end if
             first = first.getNext();
         }//while
-        if ( conflicts.length() > 0 ) throw new ChannelException(sm.getString("groupChannel.optionFlag.conflict",
-                conflicts.toString()));
+        if ( conflicts.length() > 0 ) {
+            throw new ChannelException(sm.getString("groupChannel.optionFlag.conflict",
+                    conflicts.toString()));
+        }
 
     }
 
@@ -466,10 +488,14 @@ public class GroupChannel extends ChannelInterceptorBase
     @Override
     public synchronized void start(int svc) throws ChannelException {
         setupDefaultStack();
-        if (optionCheck) checkOptionFlags();
+        if (optionCheck) {
+            checkOptionFlags();
+        }
         // register jmx
         JmxRegistry jmxRegistry = JmxRegistry.getRegistry(this);
-        if (jmxRegistry != null) this.oname = jmxRegistry.registerJmx(",component=Channel", this);
+        if (jmxRegistry != null) {
+            this.oname = jmxRegistry.registerJmx(",component=Channel", this);
+        }
         if (utilityExecutor == null) {
             log.warn(sm.getString("groupChannel.warn.noUtilityExecutor"));
             utilityExecutor = new ScheduledThreadPoolExecutor(1);
@@ -527,8 +553,11 @@ public class GroupChannel extends ChannelInterceptorBase
      * @return ChannelInterceptor
      */
     public ChannelInterceptor getFirstInterceptor() {
-        if (interceptors != null) return interceptors;
-        else return coordinator;
+        if (interceptors != null) {
+            return interceptors;
+        } else {
+            return coordinator;
+        }
     }
 
     @Override
@@ -602,8 +631,9 @@ public class GroupChannel extends ChannelInterceptorBase
      */
     @Override
     public void addMembershipListener(MembershipListener membershipListener) {
-        if (!this.membershipListeners.contains(membershipListener) )
+        if (!this.membershipListeners.contains(membershipListener) ) {
             this.membershipListeners.add(membershipListener);
+        }
     }
 
     /**
