@@ -5,9 +5,9 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -29,15 +29,15 @@ import java.util.concurrent.atomic.AtomicInteger;
  * session replication.
  */
 public class TcclThreadFactory implements ThreadFactory {
-    
+
     private static final AtomicInteger poolNumber = new AtomicInteger(1);
     private static final boolean IS_SECURITY_ENABLED =
         (System.getSecurityManager() != null);
-    
+
     private final ThreadGroup group;
     private final AtomicInteger threadNumber = new AtomicInteger(1);
     private final String namePrefix;
-    
+
     public TcclThreadFactory() {
         this("pool-" + poolNumber.getAndIncrement() + "-thread-");
     }
@@ -50,9 +50,8 @@ public class TcclThreadFactory implements ThreadFactory {
 
     @Override
     public Thread newThread(Runnable r) {
-        final Thread t = new Thread(group, r, namePrefix +
-                threadNumber.getAndIncrement());
-        
+        final Thread t = new Thread(group, r, namePrefix + threadNumber.getAndIncrement());
+
         if (IS_SECURITY_ENABLED) {
             AccessController.doPrivileged(new PrivilegedAction<Void>() {
                 @Override
