@@ -44,6 +44,8 @@ public class Introspection {
      *
      * Note: This method assumes that the method name has already been checked
      *       for correctness.
+     * @param setter The setter method
+     * @return the bean property name
      */
     public static String getPropertyName(Method setter) {
         return Introspector.decapitalize(setter.getName().substring(3));
@@ -91,17 +93,13 @@ public class Introspection {
     /**
      * Obtain the declared fields for a class taking account of any security
      * manager that may be configured.
+     * @param clazz The class to introspect
+     * @return the class fields as an array
      */
     public static Field[] getDeclaredFields(final Class<?> clazz) {
         Field[] fields = null;
         if (Globals.IS_SECURITY_ENABLED) {
-            fields = AccessController.doPrivileged(
-                    new PrivilegedAction<Field[]>(){
-                @Override
-                public Field[] run(){
-                    return clazz.getDeclaredFields();
-                }
-            });
+            fields = AccessController.doPrivileged((PrivilegedAction<Field[]>) clazz::getDeclaredFields);
         } else {
             fields = clazz.getDeclaredFields();
         }
@@ -112,17 +110,13 @@ public class Introspection {
     /**
      * Obtain the declared methods for a class taking account of any security
      * manager that may be configured.
+     * @param clazz The class to introspect
+     * @return the class methods as an array
      */
     public static Method[] getDeclaredMethods(final Class<?> clazz) {
         Method[] methods = null;
         if (Globals.IS_SECURITY_ENABLED) {
-            methods = AccessController.doPrivileged(
-                    new PrivilegedAction<Method[]>(){
-                @Override
-                public Method[] run(){
-                    return clazz.getDeclaredMethods();
-                }
-            });
+            methods = AccessController.doPrivileged((PrivilegedAction<Method[]>) clazz::getDeclaredMethods);
         } else {
             methods = clazz.getDeclaredMethods();
         }
@@ -134,6 +128,10 @@ public class Introspection {
      * Attempt to load a class using the given Container's class loader. If the
      * class cannot be loaded, a debug level log message will be written to the
      * Container's log and null will be returned.
+     * @param context The class loader of this context will be used to attempt
+     *  to load the class
+     * @param className The class name
+     * @return the loaded class or <code>null</code> if loading failed
      */
     public static Class<?> loadClass(Context context, String className) {
         ClassLoader cl = context.getLoader().getClassLoader();
@@ -141,11 +139,7 @@ public class Introspection {
         Class<?> clazz = null;
         try {
             clazz = cl.loadClass(className);
-        } catch (ClassNotFoundException e) {
-            log.debug(sm.getString("introspection.classLoadFailed", className), e);
-        } catch (NoClassDefFoundError e) {
-            log.debug(sm.getString("introspection.classLoadFailed", className), e);
-        } catch (ClassFormatError e) {
+        } catch (ClassNotFoundException | NoClassDefFoundError | ClassFormatError e) {
             log.debug(sm.getString("introspection.classLoadFailed", className), e);
         } catch (Throwable t) {
             ExceptionUtils.handleThrowable(t);
