@@ -16,7 +16,6 @@
  */
 package org.apache.catalina.valves.rewrite;
 
-import java.util.Locale;
 import java.util.Map;
 import java.util.regex.Matcher;
 import java.util.regex.Pattern;
@@ -29,18 +28,22 @@ public class RewriteCond {
 
     public static class PatternCondition extends Condition {
         public Pattern pattern;
-        public Matcher matcher = null;
+        private ThreadLocal<Matcher> matcher = new ThreadLocal<>();
 
         @Override
         public boolean evaluate(String value, Resolver resolver) {
             Matcher m = pattern.matcher(value);
             if (m.matches()) {
-                matcher = m;
+                matcher.set(m);
                 return true;
             } else {
                 return false;
             }
         }
+
+        public Matcher getMatcher() {
+            return matcher.get();
+        }
     }
 
     public static class LexicalCondition extends Condition {
@@ -85,6 +88,7 @@ public class RewriteCond {
 
     protected String testString = null;
     protected String condPattern = null;
+    protected String flagsString = null;
 
     public String getCondPattern() {
         return condPattern;
@@ -102,6 +106,14 @@ public class RewriteCond {
         this.testString = testString;
     }
 
+    public final String getFlagsString() {
+        return flagsString;
+    }
+
+    public final void setFlagsString(String flagsString) {
+        this.flagsString = flagsString;
+    }
+
     public void parse(Map<String, RewriteMap> maps) {
         test = new Substitution();
         test.setSub(testString);
@@ -111,40 +123,46 @@ public class RewriteCond {
             condPattern = condPattern.substring(1);
         }
         if (condPattern.startsWith("<")) {
-            LexicalCondition condition = new LexicalCondition();
-            condition.type = -1;
-            condition.condition = condPattern.substring(1);
+            LexicalCondition ncondition = new LexicalCondition();
+            ncondition.type = -1;
+            ncondition.condition = condPattern.substring(1);
+            this.condition = ncondition;
         } else if (condPattern.startsWith(">")) {
-            LexicalCondition condition = new LexicalCondition();
-            condition.type = 1;
-            condition.condition = condPattern.substring(1);
+            LexicalCondition ncondition = new LexicalCondition();
+            ncondition.type = 1;
+            ncondition.condition = condPattern.substring(1);
+            this.condition = ncondition;
         } else if (condPattern.startsWith("=")) {
-            LexicalCondition condition = new LexicalCondition();
-            condition.type = 0;
-            condition.condition = condPattern.substring(1);
+            LexicalCondition ncondition = new LexicalCondition();
+            ncondition.type = 0;
+            ncondition.condition = condPattern.substring(1);
+            this.condition = ncondition;
         } else if (condPattern.equals("-d")) {
             ResourceCondition ncondition = new ResourceCondition();
             ncondition.type = 0;
+            this.condition = ncondition;
         } else if (condPattern.equals("-f")) {
             ResourceCondition ncondition = new ResourceCondition();
             ncondition.type = 1;
+            this.condition = ncondition;
         } else if (condPattern.equals("-s")) {
             ResourceCondition ncondition = new ResourceCondition();
             ncondition.type = 2;
+            this.condition = ncondition;
         } else {
-            PatternCondition condition = new PatternCondition();
+            PatternCondition ncondition = new PatternCondition();
             int flags = 0;
             if (isNocase()) {
                 flags |= Pattern.CASE_INSENSITIVE;
             }
-            condition.pattern = Pattern.compile(condPattern, flags);
+            ncondition.pattern = Pattern.compile(condPattern, flags);
+            this.condition = ncondition;
         }
     }
 
     public Matcher getMatcher() {
-        Object condition = this.condition.get();
         if (condition instanceof PatternCondition) {
-            return ((PatternCondition) condition).matcher;
+            return ((PatternCondition) condition).getMatcher();
         }
         return null;
     }
@@ -154,8 +172,8 @@ public class RewriteCond {
      */
     @Override
     public String toString() {
-        // FIXME: Add flags if possible
-        return "RewriteCond " + testString + " " + condPattern;
+        return "RewriteCond " + testString + " " + condPattern
+                + ((flagsString != null) ? (" " + flagsString) : "");
     }
 
 
@@ -163,7 +181,7 @@ public class RewriteCond {
 
     protected Substitution test = null;
 
-    protected ThreadLocal<Condition> condition = new ThreadLocal<>();
+    protected Condition condition = null;
 
     /**
      * This makes the test case-insensitive, i.e., there is no difference between
@@ -183,52 +201,11 @@ public class RewriteCond {
      *
      * @param rule corresponding matched rule
      * @param cond last matched condition
+     * @param resolver Property resolver
+     * @return <code>true</code> if the condition matches
      */
     public boolean evaluate(Matcher rule, Matcher cond, Resolver resolver) {
         String value = test.evaluate(rule, cond, resolver);
-        if (nocase) {
-            value = value.toLowerCase(Locale.ENGLISH);
-        }
-        Condition condition = this.condition.get();
-        if (condition == null) {
-            if (condPattern.startsWith("<")) {
-                LexicalCondition ncondition = new LexicalCondition();
-                ncondition.type = -1;
-                ncondition.condition = condPattern.substring(1);
-                condition = ncondition;
-            } else if (condPattern.startsWith(">")) {
-                LexicalCondition ncondition = new LexicalCondition();
-                ncondition.type = 1;
-                ncondition.condition = condPattern.substring(1);
-                condition = ncondition;
-            } else if (condPattern.startsWith("=")) {
-                LexicalCondition ncondition = new LexicalCondition();
-                ncondition.type = 0;
-                ncondition.condition = condPattern.substring(1);
-                condition = ncondition;
-            } else if (condPattern.equals("-d")) {
-                ResourceCondition ncondition = new ResourceCondition();
-                ncondition.type = 0;
-                condition = ncondition;
-            } else if (condPattern.equals("-f")) {
-                ResourceCondition ncondition = new ResourceCondition();
-                ncondition.type = 1;
-                condition = ncondition;
-            } else if (condPattern.equals("-s")) {
-                ResourceCondition ncondition = new ResourceCondition();
-                ncondition.type = 2;
-                condition = ncondition;
-            } else {
-                PatternCondition ncondition = new PatternCondition();
-                int flags = 0;
-                if (isNocase()) {
-                    flags |= Pattern.CASE_INSENSITIVE;
-                }
-                ncondition.pattern = Pattern.compile(condPattern, flags);
-                condition = ncondition;
-            }
-            this.condition.set(condition);
-        }
         if (positive) {
             return condition.evaluate(value, resolver);
         } else {
