@@ -207,6 +207,12 @@ public abstract class AbstractProtocol<S> implements ProtocolHandler,
     }
 
 
+    @Override
+    public String getId() {
+        return endpoint.getId();
+    }
+
+
     // ---------------------- Properties that are passed through to the EndPoint
 
     @Override
@@ -370,9 +376,9 @@ public abstract class AbstractProtocol<S> implements ProtocolHandler,
     private String getNameInternal() {
         StringBuilder name = new StringBuilder(getNamePrefix());
         name.append('-');
-        String path = getProperty("unixDomainSocketPath");
-        if (path != null) {
-            name.append(path);
+        String id = getId();
+        if (id != null) {
+            name.append(id);
         } else {
             if (getAddress() != null) {
                 name.append(getAddress().getHostAddress());
@@ -608,9 +614,7 @@ public abstract class AbstractProtocol<S> implements ProtocolHandler,
         endpoint.start();
         monitorFuture = getUtilityExecutor().scheduleWithFixedDelay(
                 () -> {
-                    if (!isPaused()) {
-                        startAsyncTimeout();
-                    }
+                    startAsyncTimeout();
                 }, 0, 60, TimeUnit.SECONDS);
     }
 
@@ -653,7 +657,6 @@ public abstract class AbstractProtocol<S> implements ProtocolHandler,
             getLog().info(sm.getString("abstractProtocolHandler.pause", getName()));
         }
 
-        stopAsyncTimeout();
         endpoint.pause();
     }
 
@@ -670,7 +673,6 @@ public abstract class AbstractProtocol<S> implements ProtocolHandler,
         }
 
         endpoint.resume();
-        startAsyncTimeout();
     }
 
 
@@ -781,6 +783,7 @@ public abstract class AbstractProtocol<S> implements ProtocolHandler,
         }
 
 
+        @SuppressWarnings("deprecation")
         @Override
         public SocketState process(SocketWrapperBase<S> wrapper, SocketEvent status) {
             if (getLog().isDebugEnabled()) {
@@ -821,8 +824,6 @@ public abstract class AbstractProtocol<S> implements ProtocolHandler,
                 return SocketState.CLOSED;
             }
 
-            ContainerThreadMarker.set();
-
             try {
                 if (processor == null) {
                     String negotiatedProtocol = wrapper.getNegotiatedProtocol();
@@ -876,6 +877,7 @@ public abstract class AbstractProtocol<S> implements ProtocolHandler,
                     }
                 }
 
+                // Can switch to non-deprecated version in Tomcat 10.1.x
                 processor.setSslSupport(
                         wrapper.getSslSupport(getProtocol().getClientCertProvider()));
 
@@ -1041,8 +1043,6 @@ public abstract class AbstractProtocol<S> implements ProtocolHandler,
                 // with "ERROR" level, so it will show up even on
                 // less-than-verbose logs.
                 getLog().error(sm.getString("abstractConnectionHandler.error"), e);
-            } finally {
-                ContainerThreadMarker.clear();
             }
 
             // Make sure socket/processor is removed from the list of current
@@ -1219,7 +1219,9 @@ public abstract class AbstractProtocol<S> implements ProtocolHandler,
                     size.incrementAndGet();
                 }
             }
-            if (!result) handler.unregister(processor);
+            if (!result) {
+                handler.unregister(processor);
+            }
             return result;
         }
 
