@@ -19,8 +19,6 @@ package org.apache.coyote;
 import java.io.IOException;
 import java.nio.ByteBuffer;
 
-import org.apache.tomcat.util.buf.ByteChunk;
-
 /**
  * Output buffer.
  *
@@ -31,23 +29,6 @@ import org.apache.tomcat.util.buf.ByteChunk;
  */
 public interface OutputBuffer {
 
-    /**
-     * Write the given data to the response. The caller owns the chunks.
-     *
-     * @param chunk data to write
-     *
-     * @return The number of bytes written which may be less than available in
-     *         the input chunk
-     *
-     * @throws IOException an underlying I/O error occurred
-     *
-     * @deprecated Unused. Will be removed in Tomcat 9. Use
-     *             {@link #doWrite(ByteBuffer)}
-     */
-    @Deprecated
-    public int doWrite(ByteChunk chunk) throws IOException;
-
-
     /**
      * Write the given data to the response. The caller owns the chunks.
      *
