@@ -14,7 +14,6 @@
  *  See the License for the specific language governing permissions and
  *  limitations under the License.
  */
-
 package org.apache.coyote.http11.filters;
 
 import java.io.IOException;
@@ -96,24 +95,6 @@ public class BufferedInputFilter implements InputFilter, ApplicationBufferHandle
         }
     }
 
-    /**
-     * Fills the given ByteChunk with the buffered request body.
-     *
-     * @deprecated Unused. Will be removed in Tomcat 9. Use
-     *             {@link #doRead(ApplicationBufferHandler)}
-     */
-    @Override
-    public int doRead(ByteChunk chunk) throws IOException {
-        if (isFinished()) {
-            return -1;
-        }
-
-        chunk.setBytes(buffered.array(), buffered.arrayOffset() + buffered.position(),
-                buffered.remaining());
-        hasRead = true;
-        return chunk.getLength();
-    }
-
     /**
      * Fills the given ByteBuffer with the buffered request body.
      */
@@ -158,7 +139,13 @@ public class BufferedInputFilter implements InputFilter, ApplicationBufferHandle
 
     @Override
     public int available() {
-        return buffered.remaining();
+        int available = buffered.remaining();
+        if (available == 0) {
+            // No data buffered here. Try the next filter in the chain.
+            return buffer.available();
+        } else {
+            return available;
+        }
     }
 
 
