@@ -53,6 +53,11 @@ enum FrameType {
     }
 
 
+    int getId() {
+        return id;
+    }
+
+
     byte getIdByte() {
         return (byte) id;
     }
