@@ -14,7 +14,6 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
-
 package org.apache.jasper.compiler;
 
 import java.util.ArrayList;
@@ -45,16 +44,18 @@ public class SmapStratum {
         private boolean lineFileIDSet = false;
 
         public void setInputStartLine(int inputStartLine) {
-            if (inputStartLine < 0)
+            if (inputStartLine < 0) {
                 throw new IllegalArgumentException(Localizer.getMessage(
                         "jsp.error.negativeParameter", Integer.valueOf(inputStartLine)));
+            }
             this.inputStartLine = inputStartLine;
         }
 
         public void setOutputStartLine(int outputStartLine) {
-            if (outputStartLine < 0)
+            if (outputStartLine < 0) {
                 throw new IllegalArgumentException(Localizer.getMessage(
                         "jsp.error.negativeParameter", Integer.valueOf(outputStartLine)));
+            }
             this.outputStartLine = outputStartLine;
         }
 
@@ -67,24 +68,27 @@ public class SmapStratum {
          * @param lineFileID The new line file ID
          */
         public void setLineFileID(int lineFileID) {
-            if (lineFileID < 0)
+            if (lineFileID < 0) {
                 throw new IllegalArgumentException(Localizer.getMessage(
                         "jsp.error.negativeParameter", Integer.valueOf(lineFileID)));
+            }
             this.lineFileID = lineFileID;
             this.lineFileIDSet = true;
         }
 
         public void setInputLineCount(int inputLineCount) {
-            if (inputLineCount < 0)
+            if (inputLineCount < 0) {
                 throw new IllegalArgumentException(Localizer.getMessage(
                         "jsp.error.negativeParameter", Integer.valueOf(inputLineCount)));
+            }
             this.inputLineCount = inputLineCount;
         }
 
         public void setOutputLineIncrement(int outputLineIncrement) {
-            if (outputLineIncrement < 0)
+            if (outputLineIncrement < 0) {
                 throw new IllegalArgumentException(Localizer.getMessage(
                         "jsp.error.negativeParameter", Integer.valueOf(outputLineIncrement)));
+            }
             this.outputLineIncrement = outputLineIncrement;
         }
 
@@ -98,17 +102,21 @@ public class SmapStratum {
          *         specified, as its necessity is sensitive to context).
          */
         public String getString() {
-            if (inputStartLine == -1 || outputStartLine == -1)
+            if (inputStartLine == -1 || outputStartLine == -1) {
                 throw new IllegalStateException();
+            }
             StringBuilder out = new StringBuilder();
             out.append(inputStartLine);
-            if (lineFileIDSet)
+            if (lineFileIDSet) {
                 out.append("#" + lineFileID);
-            if (inputLineCount != 1)
+            }
+            if (inputLineCount != 1) {
                 out.append("," + inputLineCount);
+            }
             out.append(":" + outputStartLine);
-            if (outputLineIncrement != 1)
+            if (outputLineIncrement != 1) {
                 out.append("," + outputLineIncrement);
+            }
             out.append('\n');
             return out.toString();
         }
@@ -244,17 +252,19 @@ public class SmapStratum {
         int outputLineIncrement) {
         // check the input - what are you doing here??
         int fileIndex = filePathList.indexOf(inputFileName);
-        if (fileIndex == -1) // still
+        if (fileIndex == -1) {
             throw new IllegalArgumentException(
                 "inputFileName: " + inputFileName);
+        }
 
         //Jasper incorrectly SMAPs certain Nodes, giving them an
         //outputStartLine of 0.  This can cause a fatal error in
         //optimizeLineSection, making it impossible for Jasper to
         //compile the JSP.  Until we can fix the underlying
         //SMAPping problem, we simply ignore the flawed SMAP entries.
-        if (outputStartLine == 0)
+        if (outputStartLine == 0) {
             return;
+        }
 
         // build the LineInfo
         LineInfo li = new LineInfo();
@@ -262,8 +272,9 @@ public class SmapStratum {
         li.setInputLineCount(inputLineCount);
         li.setOutputStartLine(outputStartLine);
         li.setOutputLineIncrement(outputLineIncrement);
-        if (fileIndex != lastFileID)
+        if (fileIndex != lastFileID) {
             li.setLineFileID(fileIndex);
+        }
         lastFileID = fileIndex;
 
         // save it
