@@ -14,8 +14,6 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
-
-
 package org.apache.naming;
 
 import java.util.Enumeration;
@@ -270,11 +268,13 @@ public class NamingContext implements Context {
             return;
         }
 
-        while ((!name.isEmpty()) && (name.get(0).length() == 0))
+        while ((!name.isEmpty()) && (name.get(0).length() == 0)) {
             name = name.getSuffix(1);
-        if (name.isEmpty())
+        }
+        if (name.isEmpty()) {
             throw new NamingException
                 (sm.getString("namingContext.invalidName"));
+        }
 
         NamingEntry entry = bindings.get(name.get(0));
 
@@ -365,8 +365,9 @@ public class NamingContext implements Context {
     public NamingEnumeration<NameClassPair> list(Name name)
         throws NamingException {
         // Removing empty parts
-        while ((!name.isEmpty()) && (name.get(0).length() == 0))
+        while ((!name.isEmpty()) && (name.get(0).length() == 0)) {
             name = name.getSuffix(1);
+        }
         if (name.isEmpty()) {
             return new NamingContextEnumeration(bindings.values().iterator());
         }
@@ -419,8 +420,9 @@ public class NamingContext implements Context {
     public NamingEnumeration<Binding> listBindings(Name name)
         throws NamingException {
         // Removing empty parts
-        while ((!name.isEmpty()) && (name.get(0).length() == 0))
+        while ((!name.isEmpty()) && (name.get(0).length() == 0)) {
             name = name.getSuffix(1);
+        }
         if (name.isEmpty()) {
             return new NamingContextBindingsEnumeration(bindings.values().iterator(), this);
         }
@@ -488,11 +490,13 @@ public class NamingContext implements Context {
             return;
         }
 
-        while ((!name.isEmpty()) && (name.get(0).length() == 0))
+        while ((!name.isEmpty()) && (name.get(0).length() == 0)) {
             name = name.getSuffix(1);
-        if (name.isEmpty())
+        }
+        if (name.isEmpty()) {
             throw new NamingException
                 (sm.getString("namingContext.invalidName"));
+        }
 
         NamingEntry entry = bindings.get(name.get(0));
 
@@ -633,10 +637,12 @@ public class NamingContext implements Context {
     public NameParser getNameParser(Name name)
         throws NamingException {
 
-        while ((!name.isEmpty()) && (name.get(0).length() == 0))
+        while ((!name.isEmpty()) && (name.get(0).length() == 0)) {
             name = name.getSuffix(1);
-        if (name.isEmpty())
+        }
+        if (name.isEmpty()) {
             return nameParser;
+        }
 
         if (name.size() > 1) {
             Object obj = bindings.get(name.get(0));
@@ -819,8 +825,9 @@ public class NamingContext implements Context {
         throws NamingException {
 
         // Removing empty parts
-        while ((!name.isEmpty()) && (name.get(0).length() == 0))
+        while ((!name.isEmpty()) && (name.get(0).length() == 0)) {
             name = name.getSuffix(1);
+        }
         if (name.isEmpty()) {
             // If name is empty, a newly allocated naming context is returned
             return new NamingContext(env, this.name, bindings);
@@ -861,7 +868,7 @@ public class NamingContext implements Context {
                         // Note: This may need manual constructor reflection configuration
                         Reference reference = (Reference) entry.value;
                         Class<?> factoryClass = getClass().getClassLoader().loadClass(reference.getFactoryClassName());
-                        ObjectFactory factory = (ObjectFactory) factoryClass.newInstance();
+                        ObjectFactory factory = (ObjectFactory) factoryClass.getDeclaredConstructor().newInstance();
                         obj = factory.getObjectInstance(entry.value, name, this, env);
                     }
                     if (entry.value instanceof ResourceRef) {
@@ -873,6 +880,9 @@ public class NamingContext implements Context {
                             entry.value = obj;
                         }
                     }
+                    if (obj == null) {
+                        throw new NamingException(sm.getString("namingContext.failResolvingReference"));
+                    }
                     return obj;
                 } catch (NamingException e) {
                     throw e;
@@ -911,11 +921,13 @@ public class NamingContext implements Context {
             return;
         }
 
-        while ((!name.isEmpty()) && (name.get(0).length() == 0))
+        while ((!name.isEmpty()) && (name.get(0).length() == 0)) {
             name = name.getSuffix(1);
-        if (name.isEmpty())
+        }
+        if (name.isEmpty()) {
             throw new NamingException
                 (sm.getString("namingContext.invalidName"));
+        }
 
         NamingEntry entry = bindings.get(name.get(0));
 
