@@ -43,12 +43,8 @@ public class Import {
     private static void processFile(File f) throws IOException {
         String language = Utils.getLanguage(f.getName());
 
-        // Skip the original
-        if (language.length() == 0) {
-            // Comment this line out if the originals needs to be exported.
-            return;
-        }
-
+        // Unlike the master branch, don't skip the original so we can import
+        // updates to the English translations
         Properties props = Utils.load(f);
         Object[] objKeys = props.keySet().toArray();
         Arrays.sort(objKeys);
@@ -59,6 +55,11 @@ public class Import {
 
         for (Object objKey : objKeys) {
             String key = (String) objKey;
+            String value = props.getProperty(key);
+            // Skip untranslated values
+            if (value.trim().length() == 0) {
+                continue;
+            }
             CompositeKey cKey = new CompositeKey(key);
 
             if (!cKey.pkg.equals(currentPkg)) {
@@ -69,7 +70,7 @@ public class Import {
                 File outFile = new File(currentPkg.replace('.', File.separatorChar), Constants.L10N_PREFIX + language + Constants.L10N_SUFFIX);
                 FileOutputStream fos = new FileOutputStream(outFile);
                 w = new OutputStreamWriter(fos, StandardCharsets.UTF_8);
-                insertLicense(w);
+                org.apache.tomcat.buildutil.Utils.insertLicense(w);
             }
 
             if (!currentGroup.equals(cKey.group)) {
@@ -77,7 +78,7 @@ public class Import {
                 w.write(System.lineSeparator());
             }
 
-            w.write(cKey.key + "=" + Utils.formatValue(props.getProperty(key)));
+            w.write(cKey.key + "=" + Utils.formatValueImport(value));
             w.write(System.lineSeparator());
         }
         if (w != null) {
@@ -86,36 +87,6 @@ public class Import {
     }
 
 
-    private static void insertLicense(Writer w) throws IOException {
-        w.write("# Licensed to the Apache Software Foundation (ASF) under one or more");
-        w.write(System.lineSeparator());
-        w.write("# contributor license agreements.  See the NOTICE file distributed with");
-        w.write(System.lineSeparator());
-        w.write("# this work for additional information regarding copyright ownership.");
-        w.write(System.lineSeparator());
-        w.write("# The ASF licenses this file to You under the Apache License, Version 2.0");
-        w.write(System.lineSeparator());
-        w.write("# (the \"License\"); you may not use this file except in compliance with");
-        w.write(System.lineSeparator());
-        w.write("# the License.  You may obtain a copy of the License at");
-        w.write(System.lineSeparator());
-        w.write("#");
-        w.write(System.lineSeparator());
-        w.write("#     http://www.apache.org/licenses/LICENSE-2.0");
-        w.write(System.lineSeparator());
-        w.write("#");
-        w.write(System.lineSeparator());
-        w.write("# Unless required by applicable law or agreed to in writing, software");
-        w.write(System.lineSeparator());
-        w.write("# distributed under the License is distributed on an \"AS IS\" BASIS,");
-        w.write(System.lineSeparator());
-        w.write("# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.");
-        w.write(System.lineSeparator());
-        w.write("# See the License for the specific language governing permissions and");
-        w.write(System.lineSeparator());
-        w.write("# limitations under the License.");
-        w.write(System.lineSeparator());
-    }
     private static class CompositeKey {
 
         public final String pkg;
