@@ -22,14 +22,15 @@ import java.lang.ref.SoftReference;
 import java.util.ArrayList;
 import java.util.Iterator;
 import java.util.NoSuchElementException;
+import java.util.Optional;
 
 import org.apache.tomcat.dbcp.pool2.BaseObjectPool;
+import org.apache.tomcat.dbcp.pool2.ObjectPool;
 import org.apache.tomcat.dbcp.pool2.PoolUtils;
 import org.apache.tomcat.dbcp.pool2.PooledObjectFactory;
 
 /**
- * A {@link java.lang.ref.SoftReference SoftReference} based
- * {@link org.apache.tomcat.dbcp.pool2.ObjectPool}.
+ * A {@link java.lang.ref.SoftReference SoftReference} based {@link ObjectPool}.
  * <p>
  * This class is intended to be thread-safe.
  * </p>
@@ -52,14 +53,14 @@ public class SoftReferenceObjectPool<T> extends BaseObjectPool<T> {
     private final ReferenceQueue<T> refQueue = new ReferenceQueue<>();
 
     /** Count of instances that have been checkout out to pool clients */
-    private int numActive = 0; // @GuardedBy("this")
+    private int numActive; // @GuardedBy("this")
 
     /** Total number of instances that have been destroyed */
-    private long destroyCount = 0; // @GuardedBy("this")
+    private long destroyCount; // @GuardedBy("this")
 
 
     /** Total number of instances that have been created */
-    private long createCount = 0; // @GuardedBy("this")
+    private long createCount; // @GuardedBy("this")
 
     /** Idle references - waiting to be borrowed */
     private final LinkedBlockingDeque<PooledSoftReference<T>> idleReferences =
@@ -70,7 +71,7 @@ public class SoftReferenceObjectPool<T> extends BaseObjectPool<T> {
         new ArrayList<>();
 
     /**
-     * Create a {@code SoftReferenceObjectPool} with the specified factory.
+     * Constructs a {@code SoftReferenceObjectPool} with the specified factory.
      *
      * @param factory object factory to use.
      */
@@ -78,6 +79,65 @@ public class SoftReferenceObjectPool<T> extends BaseObjectPool<T> {
         this.factory = factory;
     }
 
+    /**
+     * Creates an object, and places it into the pool. addObject() is useful for
+     * "pre-loading" a pool with idle objects.
+     * <p>
+     * Before being added to the pool, the newly created instance is
+     * {@link PooledObjectFactory#validateObject(
+     * org.apache.tomcat.dbcp.pool2.PooledObject) validated} and
+     * {@link PooledObjectFactory#passivateObject(
+     * org.apache.tomcat.dbcp.pool2.PooledObject) passivated}. If
+     * validation fails, the new instance is
+     * {@link PooledObjectFactory#destroyObject(
+     * org.apache.tomcat.dbcp.pool2.PooledObject) destroyed}. Exceptions
+     * generated by the factory {@code makeObject} or
+     * {@code passivate} are propagated to the caller. Exceptions
+     * destroying instances are silently swallowed.
+     * </p>
+     *
+     * @throws IllegalStateException
+     *             if invoked on a {@link #close() closed} pool
+     * @throws Exception
+     *             when the {@link #getFactory() factory} has a problem creating
+     *             or passivating an object.
+     */
+    @Override
+    public synchronized void addObject() throws Exception {
+        assertOpen();
+        if (factory == null) {
+            throw new IllegalStateException(
+                    "Cannot add objects without a factory.");
+        }
+        final T obj = factory.makeObject().getObject();
+        createCount++;
+        // Create and register with the queue
+        final PooledSoftReference<T> ref = new PooledSoftReference<>(
+                new SoftReference<>(obj, refQueue));
+        allReferences.add(ref);
+
+        boolean success = true;
+        if (!factory.validateObject(ref)) {
+            success = false;
+        } else {
+            factory.passivateObject(ref);
+        }
+
+        final boolean shouldDestroy = !success;
+        if (success) {
+            idleReferences.add(ref);
+            notifyAll(); // numActive has changed
+        }
+
+        if (shouldDestroy) {
+            try {
+                destroy(ref);
+            } catch (final Exception e) {
+                // ignored
+            }
+        }
+    }
+
     /**
      * Borrows an object from the pool. If there are no idle instances available
      * in the pool, the configured factory's
@@ -103,6 +163,7 @@ public class SoftReferenceObjectPool<T> extends BaseObjectPool<T> {
      * {@code ThreadDeath} or {@code VirtualMachineError}, exceptions
      * generated by activation, validation or destroy methods are swallowed
      * silently.
+     * </p>
      *
      * @throws NoSuchElementException
      *             if a valid object cannot be provided
@@ -156,9 +217,7 @@ public class SoftReferenceObjectPool<T> extends BaseObjectPool<T> {
                         obj = null;
                     }
                     if (newlyCreated) {
-                        throw new NoSuchElementException(
-                                "Could not create a validated object, cause: " +
-                                        t.getMessage());
+                        throw new NoSuchElementException("Could not create a validated object, cause: " + t);
                     }
                 }
             }
@@ -168,6 +227,149 @@ public class SoftReferenceObjectPool<T> extends BaseObjectPool<T> {
         return obj;
     }
 
+    /**
+     * Clears any objects sitting idle in the pool.
+     */
+    @Override
+    public synchronized void clear() {
+        if (null != factory) {
+            idleReferences.forEach(ref -> {
+                try {
+                    if (null != ref.getObject()) {
+                        factory.destroyObject(ref);
+                    }
+                } catch (final Exception e) {
+                    // ignore error, keep destroying the rest
+                }
+            });
+        }
+        idleReferences.clear();
+        pruneClearedReferences();
+    }
+
+    /**
+     * Closes this pool, and frees any resources associated with it. Invokes
+     * {@link #clear()} to destroy and remove instances in the pool.
+     * <p>
+     * Calling {@link #addObject} or {@link #borrowObject} after invoking this
+     * method on a pool will cause them to throw an
+     * {@link IllegalStateException}.
+     * </p>
+     */
+    @Override
+    public void close() {
+        super.close();
+        clear();
+    }
+
+    /**
+     * Destroys a {@code PooledSoftReference} and removes it from the idle and all
+     * references pools.
+     *
+     * @param toDestroy PooledSoftReference to destroy
+     *
+     * @throws Exception If an error occurs while trying to destroy the object
+     */
+    private void destroy(final PooledSoftReference<T> toDestroy) throws Exception {
+        toDestroy.invalidate();
+        idleReferences.remove(toDestroy);
+        allReferences.remove(toDestroy);
+        try {
+            factory.destroyObject(toDestroy);
+        } finally {
+            destroyCount++;
+            toDestroy.getReference().clear();
+        }
+    }
+
+    /**
+     * Finds the PooledSoftReference in allReferences that points to obj.
+     *
+     * @param obj returning object
+     * @return PooledSoftReference wrapping a soft reference to obj
+     */
+    private PooledSoftReference<T> findReference(final T obj) {
+        final Optional<PooledSoftReference<T>> first = allReferences.stream()
+                .filter(reference -> reference.getObject() != null && reference.getObject().equals(obj)).findFirst();
+        return first.orElse(null);
+    }
+
+    /**
+     * Gets the {@link PooledObjectFactory} used by this pool to create and
+     * manage object instances.
+     *
+     * @return the factory
+     */
+    public synchronized PooledObjectFactory<T> getFactory() {
+        return factory;
+    }
+
+    /**
+     * Gets the number of instances currently borrowed from this pool.
+     *
+     * @return the number of instances currently borrowed from this pool
+     */
+    @Override
+    public synchronized int getNumActive() {
+        return numActive;
+    }
+
+    /**
+     * Gets an approximation not less than the of the number of idle
+     * instances in the pool.
+     *
+     * @return estimated number of idle instances in the pool
+     */
+    @Override
+    public synchronized int getNumIdle() {
+        pruneClearedReferences();
+        return idleReferences.size();
+    }
+
+    /**
+     * {@inheritDoc}
+     */
+    @Override
+    public synchronized void invalidateObject(final T obj) throws Exception {
+        final PooledSoftReference<T> ref = findReference(obj);
+        if (ref == null) {
+            throw new IllegalStateException(
+                "Object to invalidate is not currently part of this pool");
+        }
+        if (factory != null) {
+            destroy(ref);
+        }
+        numActive--;
+        notifyAll(); // numActive has changed
+    }
+
+    /**
+     * If any idle objects were garbage collected, remove their
+     * {@link Reference} wrappers from the idle object pool.
+     */
+    private void pruneClearedReferences() {
+        // Remove wrappers for enqueued references from idle and allReferences lists
+        removeClearedReferences(idleReferences.iterator());
+        removeClearedReferences(allReferences.iterator());
+        while (refQueue.poll() != null) {
+            // empty
+        }
+    }
+
+    /**
+     * Clears cleared references from iterator's collection
+     * @param iterator iterator over idle/allReferences
+     */
+    private void removeClearedReferences(final Iterator<PooledSoftReference<T>> iterator) {
+        PooledSoftReference<T> ref;
+        while (iterator.hasNext()) {
+            ref = iterator.next();
+            if (ref.getReference() == null || ref.getReference().isEnqueued()) {
+                iterator.remove();
+            }
+        }
+    }
+
     /**
      * Returns an instance to the pool after successful validation and
      * passivation. The returning instance is destroyed if any of the following
@@ -228,210 +430,6 @@ public class SoftReferenceObjectPool<T> extends BaseObjectPool<T> {
         }
     }
 
-    /**
-     * {@inheritDoc}
-     */
-    @Override
-    public synchronized void invalidateObject(final T obj) throws Exception {
-        final PooledSoftReference<T> ref = findReference(obj);
-        if (ref == null) {
-            throw new IllegalStateException(
-                "Object to invalidate is not currently part of this pool");
-        }
-        if (factory != null) {
-            destroy(ref);
-        }
-        numActive--;
-        notifyAll(); // numActive has changed
-    }
-
-    /**
-     * Creates an object, and places it into the pool. addObject() is useful for
-     * "pre-loading" a pool with idle objects.
-     * <p>
-     * Before being added to the pool, the newly created instance is
-     * {@link PooledObjectFactory#validateObject(
-     * org.apache.tomcat.dbcp.pool2.PooledObject) validated} and
-     * {@link PooledObjectFactory#passivateObject(
-     * org.apache.tomcat.dbcp.pool2.PooledObject) passivated}. If
-     * validation fails, the new instance is
-     * {@link PooledObjectFactory#destroyObject(
-     * org.apache.tomcat.dbcp.pool2.PooledObject) destroyed}. Exceptions
-     * generated by the factory {@code makeObject} or
-     * {@code passivate} are propagated to the caller. Exceptions
-     * destroying instances are silently swallowed.
-     *
-     * @throws IllegalStateException
-     *             if invoked on a {@link #close() closed} pool
-     * @throws Exception
-     *             when the {@link #getFactory() factory} has a problem creating
-     *             or passivating an object.
-     */
-    @Override
-    public synchronized void addObject() throws Exception {
-        assertOpen();
-        if (factory == null) {
-            throw new IllegalStateException(
-                    "Cannot add objects without a factory.");
-        }
-        final T obj = factory.makeObject().getObject();
-        createCount++;
-        // Create and register with the queue
-        final PooledSoftReference<T> ref = new PooledSoftReference<>(
-                new SoftReference<>(obj, refQueue));
-        allReferences.add(ref);
-
-        boolean success = true;
-        if (!factory.validateObject(ref)) {
-            success = false;
-        } else {
-            factory.passivateObject(ref);
-        }
-
-        final boolean shouldDestroy = !success;
-        if (success) {
-            idleReferences.add(ref);
-            notifyAll(); // numActive has changed
-        }
-
-        if (shouldDestroy) {
-            try {
-                destroy(ref);
-            } catch (final Exception e) {
-                // ignored
-            }
-        }
-    }
-
-    /**
-     * Returns an approximation not less than the of the number of idle
-     * instances in the pool.
-     *
-     * @return estimated number of idle instances in the pool
-     */
-    @Override
-    public synchronized int getNumIdle() {
-        pruneClearedReferences();
-        return idleReferences.size();
-    }
-
-    /**
-     * Returns the number of instances currently borrowed from this pool.
-     *
-     * @return the number of instances currently borrowed from this pool
-     */
-    @Override
-    public synchronized int getNumActive() {
-        return numActive;
-    }
-
-    /**
-     * Clears any objects sitting idle in the pool.
-     */
-    @Override
-    public synchronized void clear() {
-        if (null != factory) {
-            for (PooledSoftReference<T> idleReference : idleReferences) {
-                try {
-                    final PooledSoftReference<T> ref = idleReference;
-                    if (null != ref.getObject()) {
-                        factory.destroyObject(ref);
-                    }
-                } catch (final Exception e) {
-                    // ignore error, keep destroying the rest
-                }
-            }
-        }
-        idleReferences.clear();
-        pruneClearedReferences();
-    }
-
-    /**
-     * Closes this pool, and frees any resources associated with it. Invokes
-     * {@link #clear()} to destroy and remove instances in the pool.
-     * <p>
-     * Calling {@link #addObject} or {@link #borrowObject} after invoking this
-     * method on a pool will cause them to throw an
-     * {@link IllegalStateException}.
-     */
-    @Override
-    public void close() {
-        super.close();
-        clear();
-    }
-
-    /**
-     * Returns the {@link PooledObjectFactory} used by this pool to create and
-     * manage object instances.
-     *
-     * @return the factory
-     */
-    public synchronized PooledObjectFactory<T> getFactory() {
-        return factory;
-    }
-
-    /**
-     * If any idle objects were garbage collected, remove their
-     * {@link Reference} wrappers from the idle object pool.
-     */
-    private void pruneClearedReferences() {
-        // Remove wrappers for enqueued references from idle and allReferences lists
-        removeClearedReferences(idleReferences.iterator());
-        removeClearedReferences(allReferences.iterator());
-        while (refQueue.poll() != null) {
-            // empty
-        }
-    }
-
-    /**
-     * Finds the PooledSoftReference in allReferences that points to obj.
-     *
-     * @param obj returning object
-     * @return PooledSoftReference wrapping a soft reference to obj
-     */
-    private PooledSoftReference<T> findReference(final T obj) {
-        for (PooledSoftReference<T> reference : allReferences) {
-            if (reference.getObject() != null && reference.getObject().equals(obj)) {
-                return reference;
-            }
-        }
-        return null;
-    }
-
-    /**
-     * Destroys a {@code PooledSoftReference} and removes it from the idle and all
-     * references pools.
-     *
-     * @param toDestroy PooledSoftReference to destroy
-     *
-     * @throws Exception If an error occurs while trying to destroy the object
-     */
-    private void destroy(final PooledSoftReference<T> toDestroy) throws Exception {
-        toDestroy.invalidate();
-        idleReferences.remove(toDestroy);
-        allReferences.remove(toDestroy);
-        try {
-            factory.destroyObject(toDestroy);
-        } finally {
-            destroyCount++;
-            toDestroy.getReference().clear();
-        }
-    }
-
-    /**
-     * Clears cleared references from iterator's collection
-     * @param iterator iterator over idle/allReferences
-     */
-    private void removeClearedReferences(final Iterator<PooledSoftReference<T>> iterator) {
-        PooledSoftReference<T> ref;
-        while (iterator.hasNext()) {
-            ref = iterator.next();
-            if (ref.getReference() == null || ref.getReference().isEnqueued()) {
-                iterator.remove();
-            }
-        }
-    }
-
     @Override
     protected void toStringAppendFields(final StringBuilder builder) {
         super.toStringAppendFields(builder);
