@@ -319,9 +319,9 @@ public class CallMethodRule extends Rule {
             // for non-stringy param types
             Object param = parameters[i];
             // Tolerate null non-primitive values
-            if(null == param && !paramTypes[i].isPrimitive())
+            if(null == param && !paramTypes[i].isPrimitive()) {
                 paramValues[i] = null;
-            else if(param instanceof String &&
+            } else if(param instanceof String &&
                     !String.class.isAssignableFrom(paramTypes[i])) {
 
                 paramValues[i] =
@@ -348,7 +348,7 @@ public class CallMethodRule extends Rule {
             sb.append(targetOffset);
             sb.append(",stackdepth=");
             sb.append(digester.getCount());
-            sb.append(")");
+            sb.append(')');
             throw new org.xml.sax.SAXException(sb.toString());
         }
 
@@ -358,26 +358,26 @@ public class CallMethodRule extends Rule {
             sb.append(digester.match);
             sb.append("} Call ");
             sb.append(target.getClass().getName());
-            sb.append(".");
+            sb.append('.');
             sb.append(methodName);
-            sb.append("(");
+            sb.append('(');
             for (int i = 0; i < paramValues.length; i++) {
                 if (i > 0) {
-                    sb.append(",");
+                    sb.append(',');
                 }
                 if (paramValues[i] == null) {
                     sb.append("null");
                 } else {
                     sb.append(paramValues[i].toString());
                 }
-                sb.append("/");
+                sb.append('/');
                 if (paramTypes[i] == null) {
                     sb.append("null");
                 } else {
                     sb.append(paramTypes[i].getName());
                 }
             }
-            sb.append(")");
+            sb.append(')');
             digester.log.debug(sb.toString());
         }
         Object result = IntrospectionUtils.callMethodN(target, methodName,
@@ -386,14 +386,16 @@ public class CallMethodRule extends Rule {
 
         StringBuilder code = digester.getGeneratedCode();
         if (code != null) {
-            code.append(digester.toVariableName(target)).append(".").append(methodName);
-            code.append("(");
+            code.append(digester.toVariableName(target)).append('.').append(methodName);
+            code.append('(');
             for (int i = 0; i < paramValues.length; i++) {
                 if (i > 0) {
-                    code.append(",");
+                    code.append(", ");
                 }
                 if (bodyText != null) {
-                    code.append("\"").append(bodyText).append("\"");
+                    code.append("\"").append(IntrospectionUtils.escape(bodyText)).append("\"");
+                } else if (paramValues[i] instanceof String) {
+                    code.append("\"").append(IntrospectionUtils.escape(paramValues[i].toString())).append("\"");
                 } else {
                     code.append(digester.toVariableName(paramValues[i]));
                 }
@@ -443,8 +445,8 @@ public class CallMethodRule extends Rule {
                 sb.append(paramTypes[i].getName());
             }
         }
-        sb.append("}");
-        sb.append("]");
+        sb.append('}');
+        sb.append(']');
         return sb.toString();
     }
-}
\ No newline at end of file
+}
