@@ -20,6 +20,7 @@ import java.io.File;
 import java.io.IOException;
 import java.io.InputStream;
 import java.io.OutputStream;
+import java.io.UncheckedIOException;
 import java.io.UnsupportedEncodingException;
 
 /**
@@ -104,8 +105,10 @@ public interface FileItem extends FileItemHeadersSupport {
      * Returns the contents of the file item as an array of bytes.
      *
      * @return The contents of the file item as an array of bytes.
+     *
+     * @throws UncheckedIOException if an I/O error occurs
      */
-    byte[] get();
+    byte[] get() throws UncheckedIOException;
 
     /**
      * Returns the contents of the file item as a String, using the specified
@@ -118,8 +121,9 @@ public interface FileItem extends FileItemHeadersSupport {
      *
      * @throws UnsupportedEncodingException if the requested character
      *                                      encoding is not available.
+     * @throws IOException if an I/O error occurs
      */
-    String getString(String encoding) throws UnsupportedEncodingException;
+    String getString(String encoding) throws UnsupportedEncodingException, IOException;
 
     /**
      * Returns the contents of the file item as a String, using the default
