@@ -26,6 +26,7 @@ import java.util.Arrays;
 import java.util.Collections;
 import java.util.HashMap;
 import java.util.HashSet;
+import java.util.LinkedHashSet;
 import java.util.Map;
 import java.util.Set;
 import java.util.stream.Collectors;
@@ -64,7 +65,7 @@ public final class ObjectReflectionPropertyInspector {
 
     private static final Set<Class<?>> getKnownClasses() throws ClassNotFoundException {
         return
-            Collections.unmodifiableSet(new HashSet<>(
+            Collections.unmodifiableSet(new LinkedHashSet<>(
                     Arrays.asList(
                         Class.forName("org.apache.catalina.authenticator.jaspic.SimpleAuthConfigProvider"),
                         Class.forName("org.apache.catalina.authenticator.jaspic.PersistentProviderRegistrations$Property"),
@@ -104,7 +105,6 @@ public final class ObjectReflectionPropertyInspector {
                         Class.forName("org.apache.tomcat.util.net.AprEndpoint"),
                         Class.forName("org.apache.tomcat.util.net.Nio2Endpoint"),
                         Class.forName("org.apache.tomcat.util.net.NioEndpoint"),
-                        Class.forName("org.apache.tomcat.util.net.NioSelectorPool"),
                         Class.forName("org.apache.tomcat.util.net.SocketProperties")
                     )
                 )
