@@ -31,7 +31,7 @@ import org.apache.tomcat.jdbc.pool.PooledConnection;
 
 /**
  * Keeps track of statements associated with a connection and invokes close upon {@link java.sql.Connection#close()}
- * Useful for applications that dont close the associated statements after being done with a connection.
+ * Useful for applications that don't close the associated statements after being done with a connection.
  *
  */
 public class StatementFinalizer extends AbstractCreateStatementInterceptor {
@@ -44,8 +44,9 @@ public class StatementFinalizer extends AbstractCreateStatementInterceptor {
     @Override
     public Object createStatement(Object proxy, Method method, Object[] args, Object statement, long time) {
         try {
-            if (statement instanceof Statement)
-                statements.add(new StatementEntry((Statement)statement));
+            if (statement instanceof Statement) {
+              statements.add(new StatementEntry((Statement)statement));
+            }
         }catch (ClassCastException x) {
             //ignore this one
         }
@@ -55,7 +56,7 @@ public class StatementFinalizer extends AbstractCreateStatementInterceptor {
     @SuppressWarnings("null") // st is not null when used
     @Override
     public void closeInvoked() {
-        while (statements.size()>0) {
+        while (!statements.isEmpty()) {
             StatementEntry ws = statements.remove(0);
             Statement st = ws.getStatement();
             boolean shallClose = false;
