@@ -312,7 +312,7 @@ public class TestMapELResolver {
         Assert.assertFalse(context.isPropertyResolved());
     }
 
-    private static enum MethodUnderTest {
+    private enum MethodUnderTest {
         GET_VALUE, SET_VALUE, GET_TYPE
     }
 }
