@@ -40,6 +40,7 @@ import org.junit.runners.Parameterized.Parameter;
 
 import org.apache.catalina.Context;
 import org.apache.catalina.Wrapper;
+import org.apache.catalina.servlets.DefaultServlet.BomConfig;
 import org.apache.catalina.startup.Tomcat;
 import org.apache.catalina.startup.TomcatBaseTest;
 import org.apache.tomcat.util.buf.B2CConverter;
@@ -109,16 +110,27 @@ public abstract class DefaultServletEncodingBaseTest extends TomcatBaseTest {
     }
 
 
-    private static boolean getExpected(String fileEncoding, boolean useBom, String targetFile,
+    private static boolean getExpected(String fileEncoding, BomConfig bomConfig, String targetFile,
             String outputEncoding, boolean callSetCharacterEncoding, boolean useWriter) {
-        if (useWriter || callSetCharacterEncoding) {
+        if (targetFile.endsWith("-bom") && !bomConfig.stripBom) {
+            /*
+             * If the target file contains a BOM and the BOM is not stripped the
+             * test should always fail because BOM will be kept untouched
+             * and may result in a different character in the output
+             *
+             * We could differentiate iso-8859-1 and cp1252 but we would need logic here and in
+             * the asserts and that would make the logic in the test as complex as the code
+             * under test.
+             */
+            return false;
+        } else if (useWriter || callSetCharacterEncoding) {
             /*
              * Using a writer or setting the output character encoding means the
              * response will specify a character set. These cases therefore
              * reduce to can the file be read with the correct encoding.
              * (Assuming any BOM is always skipped in the included output.)
              */
-            if (targetFile.endsWith("-bom") && useBom ||
+            if (targetFile.endsWith("-bom") && bomConfig.useBomEncoding ||
                     targetFile.startsWith(fileEncoding) ||
                     targetFile.equals("cp1252") && fileEncoding.equals("iso-8859-1") ||
                     targetFile.equals("iso-8859-1") && fileEncoding.equals("cp1252")) {
@@ -160,7 +172,7 @@ public abstract class DefaultServletEncodingBaseTest extends TomcatBaseTest {
     public boolean useWriter;
 
 
-    protected abstract boolean getUseBom();
+    protected abstract BomConfig getUseBom();
 
 
     @Test
@@ -176,7 +188,7 @@ public abstract class DefaultServletEncodingBaseTest extends TomcatBaseTest {
         ctxt.setResponseCharacterEncoding(contextResponseEncoding);
         Wrapper defaultServlet = Tomcat.addServlet(ctxt, "default", DefaultServlet.class.getName());
         defaultServlet.addInitParameter("fileEncoding", fileEncoding);
-        defaultServlet.addInitParameter("useBomIfPresent", Boolean.toString(getUseBom()));
+        defaultServlet.addInitParameter("useBomIfPresent", getUseBom().configurationValue);
 
         ctxt.addServletMappingDecoded("/", "default");
 
@@ -214,10 +226,10 @@ public abstract class DefaultServletEncodingBaseTest extends TomcatBaseTest {
         }
         String body = res.toString();
         /*
-         * Remove BOM before checking content
-         * BOM (should be) removed by Tomcat when file is included
+         * Remove BOM before checking content if DefaultServlet is configured to
+         * remove BOM.
          */
-        if (!useInclude && targetFile.endsWith("-bom")) {
+        if (!useInclude && targetFile.endsWith("-bom") && getUseBom().stripBom) {
             body = body.substring(1);
         }
 
