@@ -26,12 +26,11 @@ import javax.servlet.http.HttpServlet;
 import javax.servlet.http.HttpServletRequest;
 import javax.servlet.http.HttpServletResponse;
 
-import static org.junit.Assert.assertEquals;
-
+import org.junit.Assert;
 import org.junit.Test;
 
 import org.apache.catalina.Context;
-import org.apache.catalina.loader.WebappClassLoader;
+import org.apache.catalina.loader.WebappClassLoaderBase;
 import org.apache.tomcat.util.buf.ByteChunk;
 
 public class TestTomcatClassLoader extends TomcatBaseTest {
@@ -44,12 +43,12 @@ public class TestTomcatClassLoader extends TomcatBaseTest {
         Context ctx = tomcat.addContext("", null);
 
         Tomcat.addServlet(ctx, "ClassLoaderReport", new ClassLoaderReport(null));
-        ctx.addServletMapping("/", "ClassLoaderReport");
+        ctx.addServletMappingDecoded("/", "ClassLoaderReport");
 
         tomcat.start();
 
         ByteChunk res = getUrl("http://localhost:" + getPort() + "/");
-        assertEquals("WEBAPP,SYSTEM,OTHER,", res.toString());
+        Assert.assertEquals("WEBAPP,SYSTEM,OTHER,", res.toString());
     }
 
     @Test
@@ -67,12 +66,12 @@ public class TestTomcatClassLoader extends TomcatBaseTest {
         Context ctx = tomcat.addContext("", null);
 
         Tomcat.addServlet(ctx, "ClassLoaderReport", new ClassLoaderReport(cl));
-        ctx.addServletMapping("/", "ClassLoaderReport");
+        ctx.addServletMappingDecoded("/", "ClassLoaderReport");
 
         tomcat.start();
 
         ByteChunk res = getUrl("http://localhost:" + getPort() + "/");
-        assertEquals("WEBAPP,CUSTOM,SYSTEM,OTHER,", res.toString());
+        Assert.assertEquals("WEBAPP,CUSTOM,SYSTEM,OTHER,", res.toString());
     }
 
     private static final class ClassLoaderReport extends HttpServlet {
@@ -98,7 +97,7 @@ public class TestTomcatClassLoader extends TomcatBaseTest {
                     out.print("SYSTEM,");
                 } else if (custom == cl) {
                     out.print("CUSTOM,");
-                } else if (cl instanceof WebappClassLoader) {
+                } else if (cl instanceof WebappClassLoaderBase) {
                     out.print("WEBAPP,");
                 } else {
                     out.print("OTHER,");
