@@ -14,7 +14,6 @@
  *  See the License for the specific language governing permissions and
  *  limitations under the License.
  */
-
 package org.apache.coyote.http11;
 
 import java.io.IOException;
@@ -198,6 +197,10 @@ public class TestHttp11InputBuffer extends TomcatBaseTest {
                 // TAB is allowed
                 continue;
             }
+            if (i == '\n') {
+                // LF is the optional line terminator
+                continue;
+            }
             doTestBug51557InvalidCharInValue((char) i);
             tearDown();
             setUp();
@@ -687,24 +690,6 @@ public class TestHttp11InputBuffer extends TomcatBaseTest {
     }
 
 
-    @Test
-    public void testInvalidEndOfRequestLine02() {
-
-        String[] request = new String[1];
-        request[0] =
-                "GET /test HTTP/1.1" + LF +
-                "Host: localhost:8080" + CRLF +
-                "Connection: close" + CRLF +
-                CRLF;
-
-        InvalidClient client = new InvalidClient(request);
-
-        client.doRequest();
-        Assert.assertTrue(client.getResponseLine(), client.isResponse400());
-        Assert.assertTrue(client.isResponseBodyOK());
-    }
-
-
     @Test
     public void testInvalidHeader01() {
 
