@@ -49,8 +49,10 @@ public class TesterOpenSSL {
         } catch (IOException e) {
             versionString = "";
         }
-        if (versionString.startsWith("OpenSSL 3.0.0")) {
-            // Note: Gump currently tests 10.x with OpenSSL 3.0.x
+        if (versionString.startsWith("OpenSSL 3.1.")) {
+            // Note: Gump currently tests 10.x with OpenSSL 3.1.x
+            VERSION = 30100;
+        } else if (versionString.startsWith("OpenSSL 3.0.")) {
             VERSION = 30000;
         } else if (versionString.startsWith("OpenSSL 1.1.1")) {
             // LTS
@@ -251,19 +253,9 @@ public class TesterOpenSSL {
         // Standard command to list the ciphers
         args.add("ciphers");
         args.add("-v");
-        if (VERSION < 10100) {
-            // Need to exclude the GOST ciphers
-            if (specification == null) {
-                specification = "DEFAULT:!aGOST";
-            } else {
-                specification = "!aGOST:" + specification;
-            }
-        }
-        if (VERSION >= 10101) {
-            // Need to exclude the TLSv1.3 ciphers
-            args.add("-ciphersuites");
-            args.add("");
-        }
+        // Need to exclude the TLSv1.3 ciphers
+        args.add("-ciphersuites");
+        args.add("");
         // Include the specification if provided
         if (specification != null) {
             args.add(specification);
@@ -337,7 +329,7 @@ public class TesterOpenSSL {
             // Explicit OpenSSL path may also need explicit lib path
             // (e.g. Gump needs this)
             openSSLLibPath = openSSLPath.substring(0, openSSLPath.lastIndexOf('/'));
-            openSSLLibPath = openSSLLibPath + "/../lib:" + openSSLLibPath + "/../lib64";
+            openSSLLibPath = openSSLLibPath + "/../:" + openSSLLibPath + "/../lib:" + openSSLLibPath + "/../lib64";
         }
         List<String> cmd = new ArrayList<>();
         cmd.add(openSSLPath);
